from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('locales', '0004_auto_20210130_0333'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userlocaleprofile',
            name='language',
            field=models.CharField(
                choices=[
                    ('ar', 'Arabic'),
                    ('bg', 'Bulgarian'),
                    ('bs', 'Bosnian'),
                    ('cs', 'Czech'),
                    ('da', 'Danish'),
                    ('de', 'German'),
                    ('el', 'Greek'),
                    ('en', 'English'),
                    ('es', 'Spanish'),
                    ('es-pr', 'Spanish (Puerto Rico)'),
                    ('fa', 'Persian'),
                    ('fr', 'French'),
                    ('hu', 'Hungarian'),
                    ('id', 'Indonesian'),
                    ('it', 'Italian'),
                    ('lv', 'Latvian'),
                    ('nl', 'Dutch'),
                    ('pl', 'Polish'),
                    ('pt', 'Portuguese'),
                    ('pt-br', 'Portuguese (Brazil)'),
                    ('ro', 'Romanian'),
                    ('ru', 'Russian'),
                    ('sl', 'Slovenian'),
                    ('tr', 'Turkish'),
                    ('vi', 'Vietnamese'),
                    ('zh-hans', 'Chinese (Simplified)')
                ], max_length=8, verbose_name='Language'),
        ),
        migrations.AlterField(
            model_name='userlocaleprofile', name='timezone',
            field=models.CharField(
                choices=[
                    ('Africa/Abidjan', 'Africa/Abidjan'),
                    ('Africa/Accra', 'Africa/Accra'),
                    ('Africa/Addis_Ababa', 'Africa/Addis_Ababa'),
                    ('Africa/Algiers', 'Africa/Algiers'),
                    ('Africa/Asmara', 'Africa/Asmara'),
                    ('Africa/Bamako', 'Africa/Bamako'),
                    ('Africa/Bangui', 'Africa/Bangui'),
                    ('Africa/Banjul', 'Africa/Banjul'),
                    ('Africa/Bissau', 'Africa/Bissau'),
                    ('Africa/Blantyre', 'Africa/Blantyre'),
                    ('Africa/Brazzaville', 'Africa/Brazzaville'),
                    ('Africa/Bujumbura', 'Africa/Bujumbura'),
                    ('Africa/Cairo', 'Africa/Cairo'),
                    ('Africa/Casablanca', 'Africa/Casablanca'),
                    ('Africa/Ceuta', 'Africa/Ceuta'),
                    ('Africa/Conakry', 'Africa/Conakry'),
                    ('Africa/Dakar', 'Africa/Dakar'),
                    ('Africa/Dar_es_Salaam', 'Africa/Dar_es_Salaam'),
                    ('Africa/Djibouti', 'Africa/Djibouti'),
                    ('Africa/Douala', 'Africa/Douala'),
                    ('Africa/El_Aaiun', 'Africa/El_Aaiun'),
                    ('Africa/Freetown', 'Africa/Freetown'),
                    ('Africa/Gaborone', 'Africa/Gaborone'),
                    ('Africa/Harare', 'Africa/Harare'),
                    ('Africa/Johannesburg', 'Africa/Johannesburg'),
                    ('Africa/Juba', 'Africa/Juba'),
                    ('Africa/Kampala', 'Africa/Kampala'),
                    ('Africa/Khartoum', 'Africa/Khartoum'),
                    ('Africa/Kigali', 'Africa/Kigali'),
                    ('Africa/Kinshasa', 'Africa/Kinshasa'),
                    ('Africa/Lagos', 'Africa/Lagos'),
                    ('Africa/Libreville', 'Africa/Libreville'),
                    ('Africa/Lome', 'Africa/Lome'),
                    ('Africa/Luanda', 'Africa/Luanda'),
                    ('Africa/Lubumbashi', 'Africa/Lubumbashi'),
                    ('Africa/Lusaka', 'Africa/Lusaka'),
                    ('Africa/Malabo', 'Africa/Malabo'),
                    ('Africa/Maputo', 'Africa/Maputo'),
                    ('Africa/Maseru', 'Africa/Maseru'),
                    ('Africa/Mbabane', 'Africa/Mbabane'),
                    ('Africa/Mogadishu', 'Africa/Mogadishu'),
                    ('Africa/Monrovia', 'Africa/Monrovia'),
                    ('Africa/Nairobi', 'Africa/Nairobi'),
                    ('Africa/Ndjamena', 'Africa/Ndjamena'),
                    ('Africa/Niamey', 'Africa/Niamey'),
                    ('Africa/Nouakchott', 'Africa/Nouakchott'),
                    ('Africa/Ouagadougou', 'Africa/Ouagadougou'),
                    ('Africa/Porto-Novo', 'Africa/Porto-Novo'),
                    ('Africa/Sao_Tome', 'Africa/Sao_Tome'),
                    ('Africa/Tripoli', 'Africa/Tripoli'),
                    ('Africa/Tunis', 'Africa/Tunis'),
                    ('Africa/Windhoek', 'Africa/Windhoek'),
                    ('America/Adak', 'America/Adak'),
                    ('America/Anchorage', 'America/Anchorage'),
                    ('America/Anguilla', 'America/Anguilla'),
                    ('America/Antigua', 'America/Antigua'),
                    ('America/Araguaina', 'America/Araguaina'),
                    ('America/Argentina/Buenos_Aires', 'America/Argentina/Buenos_Aires'),
                    ('America/Argentina/Catamarca', 'America/Argentina/Catamarca'),
                    ('America/Argentina/Cordoba', 'America/Argentina/Cordoba'),
                    ('America/Argentina/Jujuy', 'America/Argentina/Jujuy'),
                    ('America/Argentina/La_Rioja', 'America/Argentina/La_Rioja'),
                    ('America/Argentina/Mendoza', 'America/Argentina/Mendoza'),
                    ('America/Argentina/Rio_Gallegos', 'America/Argentina/Rio_Gallegos'),
                    ('America/Argentina/Salta', 'America/Argentina/Salta'),
                    ('America/Argentina/San_Juan', 'America/Argentina/San_Juan'),
                    ('America/Argentina/San_Luis', 'America/Argentina/San_Luis'),
                    ('America/Argentina/Tucuman', 'America/Argentina/Tucuman'),
                    ('America/Argentina/Ushuaia', 'America/Argentina/Ushuaia'),
                    ('America/Aruba', 'America/Aruba'),
                    ('America/Asuncion', 'America/Asuncion'),
                    ('America/Atikokan', 'America/Atikokan'),
                    ('America/Bahia', 'America/Bahia'),
                    ('America/Bahia_Banderas', 'America/Bahia_Banderas'),
                    ('America/Barbados', 'America/Barbados'),
                    ('America/Belem', 'America/Belem'),
                    ('America/Belize', 'America/Belize'),
                    ('America/Blanc-Sablon', 'America/Blanc-Sablon'),
                    ('America/Boa_Vista', 'America/Boa_Vista'),
                    ('America/Bogota', 'America/Bogota'),
                    ('America/Boise', 'America/Boise'),
                    ('America/Cambridge_Bay', 'America/Cambridge_Bay'),
                    ('America/Campo_Grande', 'America/Campo_Grande'),
                    ('America/Cancun', 'America/Cancun'),
                    ('America/Caracas', 'America/Caracas'),
                    ('America/Cayenne', 'America/Cayenne'),
                    ('America/Cayman', 'America/Cayman'),
                    ('America/Chicago', 'America/Chicago'),
                    ('America/Chihuahua', 'America/Chihuahua'),
                    ('America/Costa_Rica', 'America/Costa_Rica'),
                    ('America/Creston', 'America/Creston'),
                    ('America/Cuiaba', 'America/Cuiaba'),
                    ('America/Curacao', 'America/Curacao'),
                    ('America/Danmarkshavn', 'America/Danmarkshavn'),
                    ('America/Dawson', 'America/Dawson'),
                    ('America/Dawson_Creek', 'America/Dawson_Creek'),
                    ('America/Denver', 'America/Denver'),
                    ('America/Detroit', 'America/Detroit'),
                    ('America/Dominica', 'America/Dominica'),
                    ('America/Edmonton', 'America/Edmonton'),
                    ('America/Eirunepe', 'America/Eirunepe'),
                    ('America/El_Salvador', 'America/El_Salvador'),
                    ('America/Fort_Nelson', 'America/Fort_Nelson'),
                    ('America/Fortaleza', 'America/Fortaleza'),
                    ('America/Glace_Bay', 'America/Glace_Bay'),
                    ('America/Goose_Bay', 'America/Goose_Bay'),
                    ('America/Grand_Turk', 'America/Grand_Turk'),
                    ('America/Grenada', 'America/Grenada'),
                    ('America/Guadeloupe', 'America/Guadeloupe'),
                    ('America/Guatemala', 'America/Guatemala'),
                    ('America/Guayaquil', 'America/Guayaquil'),
                    ('America/Guyana', 'America/Guyana'),
                    ('America/Halifax', 'America/Halifax'),
                    ('America/Havana', 'America/Havana'),
                    ('America/Hermosillo', 'America/Hermosillo'),
                    ('America/Indiana/Indianapolis', 'America/Indiana/Indianapolis'),
                    ('America/Indiana/Knox', 'America/Indiana/Knox'),
                    ('America/Indiana/Marengo', 'America/Indiana/Marengo'),
                    ('America/Indiana/Petersburg', 'America/Indiana/Petersburg'),
                    ('America/Indiana/Tell_City', 'America/Indiana/Tell_City'),
                    ('America/Indiana/Vevay', 'America/Indiana/Vevay'),
                    ('America/Indiana/Vincennes', 'America/Indiana/Vincennes'),
                    ('America/Indiana/Winamac', 'America/Indiana/Winamac'),
                    ('America/Inuvik', 'America/Inuvik'),
                    ('America/Iqaluit', 'America/Iqaluit'),
                    ('America/Jamaica', 'America/Jamaica'),
                    ('America/Juneau', 'America/Juneau'),
                    ('America/Kentucky/Louisville', 'America/Kentucky/Louisville'),
                    ('America/Kentucky/Monticello', 'America/Kentucky/Monticello'),
                    ('America/Kralendijk', 'America/Kralendijk'),
                    ('America/La_Paz', 'America/La_Paz'),
                    ('America/Lima', 'America/Lima'),
                    ('America/Los_Angeles', 'America/Los_Angeles'),
                    ('America/Lower_Princes', 'America/Lower_Princes'),
                    ('America/Maceio', 'America/Maceio'),
                    ('America/Managua', 'America/Managua'),
                    ('America/Manaus', 'America/Manaus'),
                    ('America/Marigot', 'America/Marigot'),
                    ('America/Martinique', 'America/Martinique'),
                    ('America/Matamoros', 'America/Matamoros'),
                    ('America/Mazatlan', 'America/Mazatlan'),
                    ('America/Menominee', 'America/Menominee'),
                    ('America/Merida', 'America/Merida'),
                    ('America/Metlakatla', 'America/Metlakatla'),
                    ('America/Mexico_City', 'America/Mexico_City'),
                    ('America/Miquelon', 'America/Miquelon'),
                    ('America/Moncton', 'America/Moncton'),
                    ('America/Monterrey', 'America/Monterrey'),
                    ('America/Montevideo', 'America/Montevideo'),
                    ('America/Montserrat', 'America/Montserrat'),
                    ('America/Nassau', 'America/Nassau'),
                    ('America/New_York', 'America/New_York'),
                    ('America/Nipigon', 'America/Nipigon'),
                    ('America/Nome', 'America/Nome'),
                    ('America/Noronha', 'America/Noronha'),
                    ('America/North_Dakota/Beulah', 'America/North_Dakota/Beulah'),
                    ('America/North_Dakota/Center', 'America/North_Dakota/Center'),
                    ('America/North_Dakota/New_Salem', 'America/North_Dakota/New_Salem'),
                    ('America/Nuuk', 'America/Nuuk'),
                    ('America/Ojinaga', 'America/Ojinaga'),
                    ('America/Panama', 'America/Panama'),
                    ('America/Pangnirtung', 'America/Pangnirtung'),
                    ('America/Paramaribo', 'America/Paramaribo'),
                    ('America/Phoenix', 'America/Phoenix'),
                    ('America/Port-au-Prince', 'America/Port-au-Prince'),
                    ('America/Port_of_Spain', 'America/Port_of_Spain'),
                    ('America/Porto_Velho', 'America/Porto_Velho'),
                    ('America/Puerto_Rico', 'America/Puerto_Rico'),
                    ('America/Punta_Arenas', 'America/Punta_Arenas'),
                    ('America/Rainy_River', 'America/Rainy_River'),
                    ('America/Rankin_Inlet', 'America/Rankin_Inlet'),
                    ('America/Recife', 'America/Recife'),
                    ('America/Regina', 'America/Regina'),
                    ('America/Resolute', 'America/Resolute'),
                    ('America/Rio_Branco', 'America/Rio_Branco'),
                    ('America/Santarem', 'America/Santarem'),
                    ('America/Santiago', 'America/Santiago'),
                    ('America/Santo_Domingo', 'America/Santo_Domingo'),
                    ('America/Sao_Paulo', 'America/Sao_Paulo'),
                    ('America/Scoresbysund', 'America/Scoresbysund'),
                    ('America/Sitka', 'America/Sitka'),
                    ('America/St_Barthelemy', 'America/St_Barthelemy'),
                    ('America/St_Johns', 'America/St_Johns'),
                    ('America/St_Kitts', 'America/St_Kitts'),
                    ('America/St_Lucia', 'America/St_Lucia'),
                    ('America/St_Thomas', 'America/St_Thomas'),
                    ('America/St_Vincent', 'America/St_Vincent'),
                    ('America/Swift_Current', 'America/Swift_Current'),
                    ('America/Tegucigalpa', 'America/Tegucigalpa'),
                    ('America/Thule', 'America/Thule'),
                    ('America/Thunder_Bay', 'America/Thunder_Bay'),
                    ('America/Tijuana', 'America/Tijuana'),
                    ('America/Toronto', 'America/Toronto'),
                    ('America/Tortola', 'America/Tortola'),
                    ('America/Vancouver', 'America/Vancouver'),
                    ('America/Whitehorse', 'America/Whitehorse'),
                    ('America/Winnipeg', 'America/Winnipeg'),
                    ('America/Yakutat', 'America/Yakutat'),
                    ('America/Yellowknife', 'America/Yellowknife'),
                    ('Antarctica/Casey', 'Antarctica/Casey'),
                    ('Antarctica/Davis', 'Antarctica/Davis'),
                    ('Antarctica/DumontDUrville', 'Antarctica/DumontDUrville'),
                    ('Antarctica/Macquarie', 'Antarctica/Macquarie'),
                    ('Antarctica/Mawson', 'Antarctica/Mawson'),
                    ('Antarctica/McMurdo', 'Antarctica/McMurdo'),
                    ('Antarctica/Palmer', 'Antarctica/Palmer'),
                    ('Antarctica/Rothera', 'Antarctica/Rothera'),
                    ('Antarctica/Syowa', 'Antarctica/Syowa'),
                    ('Antarctica/Troll', 'Antarctica/Troll'),
                    ('Antarctica/Vostok', 'Antarctica/Vostok'),
                    ('Arctic/Longyearbyen', 'Arctic/Longyearbyen'),
                    ('Asia/Aden', 'Asia/Aden'),
                    ('Asia/Almaty', 'Asia/Almaty'),
                    ('Asia/Amman', 'Asia/Amman'),
                    ('Asia/Anadyr', 'Asia/Anadyr'),
                    ('Asia/Aqtau', 'Asia/Aqtau'),
                    ('Asia/Aqtobe', 'Asia/Aqtobe'),
                    ('Asia/Ashgabat', 'Asia/Ashgabat'),
                    ('Asia/Atyrau', 'Asia/Atyrau'),
                    ('Asia/Baghdad', 'Asia/Baghdad'),
                    ('Asia/Bahrain', 'Asia/Bahrain'),
                    ('Asia/Baku', 'Asia/Baku'),
                    ('Asia/Bangkok', 'Asia/Bangkok'),
                    ('Asia/Barnaul', 'Asia/Barnaul'),
                    ('Asia/Beirut', 'Asia/Beirut'),
                    ('Asia/Bishkek', 'Asia/Bishkek'),
                    ('Asia/Brunei', 'Asia/Brunei'),
                    ('Asia/Chita', 'Asia/Chita'),
                    ('Asia/Choibalsan', 'Asia/Choibalsan'),
                    ('Asia/Colombo', 'Asia/Colombo'),
                    ('Asia/Damascus', 'Asia/Damascus'),
                    ('Asia/Dhaka', 'Asia/Dhaka'),
                    ('Asia/Dili', 'Asia/Dili'),
                    ('Asia/Dubai', 'Asia/Dubai'),
                    ('Asia/Dushanbe', 'Asia/Dushanbe'),
                    ('Asia/Famagusta', 'Asia/Famagusta'),
                    ('Asia/Gaza', 'Asia/Gaza'),
                    ('Asia/Hebron', 'Asia/Hebron'),
                    ('Asia/Ho_Chi_Minh', 'Asia/Ho_Chi_Minh'),
                    ('Asia/Hong_Kong', 'Asia/Hong_Kong'),
                    ('Asia/Hovd', 'Asia/Hovd'),
                    ('Asia/Irkutsk', 'Asia/Irkutsk'),
                    ('Asia/Jakarta', 'Asia/Jakarta'),
                    ('Asia/Jayapura', 'Asia/Jayapura'),
                    ('Asia/Jerusalem', 'Asia/Jerusalem'),
                    ('Asia/Kabul', 'Asia/Kabul'),
                    ('Asia/Kamchatka', 'Asia/Kamchatka'),
                    ('Asia/Karachi', 'Asia/Karachi'),
                    ('Asia/Kathmandu', 'Asia/Kathmandu'),
                    ('Asia/Khandyga', 'Asia/Khandyga'),
                    ('Asia/Kolkata', 'Asia/Kolkata'),
                    ('Asia/Krasnoyarsk', 'Asia/Krasnoyarsk'),
                    ('Asia/Kuala_Lumpur', 'Asia/Kuala_Lumpur'),
                    ('Asia/Kuching', 'Asia/Kuching'),
                    ('Asia/Kuwait', 'Asia/Kuwait'),
                    ('Asia/Macau', 'Asia/Macau'),
                    ('Asia/Magadan', 'Asia/Magadan'),
                    ('Asia/Makassar', 'Asia/Makassar'),
                    ('Asia/Manila', 'Asia/Manila'),
                    ('Asia/Muscat', 'Asia/Muscat'),
                    ('Asia/Nicosia', 'Asia/Nicosia'),
                    ('Asia/Novokuznetsk', 'Asia/Novokuznetsk'),
                    ('Asia/Novosibirsk', 'Asia/Novosibirsk'),
                    ('Asia/Omsk', 'Asia/Omsk'),
                    ('Asia/Oral', 'Asia/Oral'),
                    ('Asia/Phnom_Penh', 'Asia/Phnom_Penh'),
                    ('Asia/Pontianak', 'Asia/Pontianak'),
                    ('Asia/Pyongyang', 'Asia/Pyongyang'),
                    ('Asia/Qatar', 'Asia/Qatar'),
                    ('Asia/Qostanay', 'Asia/Qostanay'),
                    ('Asia/Qyzylorda', 'Asia/Qyzylorda'),
                    ('Asia/Riyadh', 'Asia/Riyadh'),
                    ('Asia/Sakhalin', 'Asia/Sakhalin'),
                    ('Asia/Samarkand', 'Asia/Samarkand'),
                    ('Asia/Seoul', 'Asia/Seoul'),
                    ('Asia/Shanghai', 'Asia/Shanghai'),
                    ('Asia/Singapore', 'Asia/Singapore'),
                    ('Asia/Srednekolymsk', 'Asia/Srednekolymsk'),
                    ('Asia/Taipei', 'Asia/Taipei'),
                    ('Asia/Tashkent', 'Asia/Tashkent'),
                    ('Asia/Tbilisi', 'Asia/Tbilisi'),
                    ('Asia/Tehran', 'Asia/Tehran'),
                    ('Asia/Thimphu', 'Asia/Thimphu'),
                    ('Asia/Tokyo', 'Asia/Tokyo'),
                    ('Asia/Tomsk', 'Asia/Tomsk'),
                    ('Asia/Ulaanbaatar', 'Asia/Ulaanbaatar'),
                    ('Asia/Urumqi', 'Asia/Urumqi'),
                    ('Asia/Ust-Nera', 'Asia/Ust-Nera'),
                    ('Asia/Vientiane', 'Asia/Vientiane'),
                    ('Asia/Vladivostok', 'Asia/Vladivostok'),
                    ('Asia/Yakutsk', 'Asia/Yakutsk'),
                    ('Asia/Yangon', 'Asia/Yangon'),
                    ('Asia/Yekaterinburg', 'Asia/Yekaterinburg'),
                    ('Asia/Yerevan', 'Asia/Yerevan'),
                    ('Atlantic/Azores', 'Atlantic/Azores'),
                    ('Atlantic/Bermuda', 'Atlantic/Bermuda'),
                    ('Atlantic/Canary', 'Atlantic/Canary'),
                    ('Atlantic/Cape_Verde', 'Atlantic/Cape_Verde'),
                    ('Atlantic/Faroe', 'Atlantic/Faroe'),
                    ('Atlantic/Madeira', 'Atlantic/Madeira'),
                    ('Atlantic/Reykjavik', 'Atlantic/Reykjavik'),
                    ('Atlantic/South_Georgia', 'Atlantic/South_Georgia'),
                    ('Atlantic/St_Helena', 'Atlantic/St_Helena'),
                    ('Atlantic/Stanley', 'Atlantic/Stanley'),
                    ('Australia/Adelaide', 'Australia/Adelaide'),
                    ('Australia/Brisbane', 'Australia/Brisbane'),
                    ('Australia/Broken_Hill', 'Australia/Broken_Hill'),
                    ('Australia/Darwin', 'Australia/Darwin'),
                    ('Australia/Eucla', 'Australia/Eucla'),
                    ('Australia/Hobart', 'Australia/Hobart'),
                    ('Australia/Lindeman', 'Australia/Lindeman'),
                    ('Australia/Lord_Howe', 'Australia/Lord_Howe'),
                    ('Australia/Melbourne', 'Australia/Melbourne'),
                    ('Australia/Perth', 'Australia/Perth'),
                    ('Australia/Sydney', 'Australia/Sydney'),
                    ('Canada/Atlantic', 'Canada/Atlantic'),
                    ('Canada/Central', 'Canada/Central'),
                    ('Canada/Eastern', 'Canada/Eastern'),
                    ('Canada/Mountain', 'Canada/Mountain'),
                    ('Canada/Newfoundland', 'Canada/Newfoundland'),
                    ('Canada/Pacific', 'Canada/Pacific'),
                    ('Europe/Amsterdam', 'Europe/Amsterdam'),
                    ('Europe/Andorra', 'Europe/Andorra'),
                    ('Europe/Astrakhan', 'Europe/Astrakhan'),
                    ('Europe/Athens', 'Europe/Athens'),
                    ('Europe/Belgrade', 'Europe/Belgrade'),
                    ('Europe/Berlin', 'Europe/Berlin'),
                    ('Europe/Bratislava', 'Europe/Bratislava'),
                    ('Europe/Brussels', 'Europe/Brussels'),
                    ('Europe/Bucharest', 'Europe/Bucharest'),
                    ('Europe/Budapest', 'Europe/Budapest'),
                    ('Europe/Busingen', 'Europe/Busingen'),
                    ('Europe/Chisinau', 'Europe/Chisinau'),
                    ('Europe/Copenhagen', 'Europe/Copenhagen'),
                    ('Europe/Dublin', 'Europe/Dublin'),
                    ('Europe/Gibraltar', 'Europe/Gibraltar'),
                    ('Europe/Guernsey', 'Europe/Guernsey'),
                    ('Europe/Helsinki', 'Europe/Helsinki'),
                    ('Europe/Isle_of_Man', 'Europe/Isle_of_Man'),
                    ('Europe/Istanbul', 'Europe/Istanbul'),
                    ('Europe/Jersey', 'Europe/Jersey'),
                    ('Europe/Kaliningrad', 'Europe/Kaliningrad'),
                    ('Europe/Kiev', 'Europe/Kiev'),
                    ('Europe/Kirov', 'Europe/Kirov'),
                    ('Europe/Lisbon', 'Europe/Lisbon'),
                    ('Europe/Ljubljana', 'Europe/Ljubljana'),
                    ('Europe/London', 'Europe/London'),
                    ('Europe/Luxembourg', 'Europe/Luxembourg'),
                    ('Europe/Madrid', 'Europe/Madrid'),
                    ('Europe/Malta', 'Europe/Malta'),
                    ('Europe/Mariehamn', 'Europe/Mariehamn'),
                    ('Europe/Minsk', 'Europe/Minsk'),
                    ('Europe/Monaco', 'Europe/Monaco'),
                    ('Europe/Moscow', 'Europe/Moscow'),
                    ('Europe/Oslo', 'Europe/Oslo'),
                    ('Europe/Paris', 'Europe/Paris'),
                    ('Europe/Podgorica', 'Europe/Podgorica'),
                    ('Europe/Prague', 'Europe/Prague'),
                    ('Europe/Riga', 'Europe/Riga'),
                    ('Europe/Rome', 'Europe/Rome'),
                    ('Europe/Samara', 'Europe/Samara'),
                    ('Europe/San_Marino', 'Europe/San_Marino'),
                    ('Europe/Sarajevo', 'Europe/Sarajevo'),
                    ('Europe/Saratov', 'Europe/Saratov'),
                    ('Europe/Simferopol', 'Europe/Simferopol'),
                    ('Europe/Skopje', 'Europe/Skopje'),
                    ('Europe/Sofia', 'Europe/Sofia'),
                    ('Europe/Stockholm', 'Europe/Stockholm'),
                    ('Europe/Tallinn', 'Europe/Tallinn'),
                    ('Europe/Tirane', 'Europe/Tirane'),
                    ('Europe/Ulyanovsk', 'Europe/Ulyanovsk'),
                    ('Europe/Uzhgorod', 'Europe/Uzhgorod'),
                    ('Europe/Vaduz', 'Europe/Vaduz'),
                    ('Europe/Vatican', 'Europe/Vatican'),
                    ('Europe/Vienna', 'Europe/Vienna'),
                    ('Europe/Vilnius', 'Europe/Vilnius'),
                    ('Europe/Volgograd', 'Europe/Volgograd'),
                    ('Europe/Warsaw', 'Europe/Warsaw'),
                    ('Europe/Zagreb', 'Europe/Zagreb'),
                    ('Europe/Zaporozhye', 'Europe/Zaporozhye'),
                    ('Europe/Zurich', 'Europe/Zurich'),
                    ('GMT', 'GMT'),
                    ('Indian/Antananarivo', 'Indian/Antananarivo'),
                    ('Indian/Chagos', 'Indian/Chagos'),
                    ('Indian/Christmas', 'Indian/Christmas'),
                    ('Indian/Cocos', 'Indian/Cocos'),
                    ('Indian/Comoro', 'Indian/Comoro'),
                    ('Indian/Kerguelen', 'Indian/Kerguelen'),
                    ('Indian/Mahe', 'Indian/Mahe'),
                    ('Indian/Maldives', 'Indian/Maldives'),
                    ('Indian/Mauritius', 'Indian/Mauritius'),
                    ('Indian/Mayotte', 'Indian/Mayotte'),
                    ('Indian/Reunion', 'Indian/Reunion'),
                    ('Pacific/Apia', 'Pacific/Apia'),
                    ('Pacific/Auckland', 'Pacific/Auckland'),
                    ('Pacific/Bougainville', 'Pacific/Bougainville'),
                    ('Pacific/Chatham', 'Pacific/Chatham'),
                    ('Pacific/Chuuk', 'Pacific/Chuuk'),
                    ('Pacific/Easter', 'Pacific/Easter'),
                    ('Pacific/Efate', 'Pacific/Efate'),
                    ('Pacific/Enderbury', 'Pacific/Enderbury'),
                    ('Pacific/Fakaofo', 'Pacific/Fakaofo'),
                    ('Pacific/Fiji', 'Pacific/Fiji'),
                    ('Pacific/Funafuti', 'Pacific/Funafuti'),
                    ('Pacific/Galapagos', 'Pacific/Galapagos'),
                    ('Pacific/Gambier', 'Pacific/Gambier'),
                    ('Pacific/Guadalcanal', 'Pacific/Guadalcanal'),
                    ('Pacific/Guam', 'Pacific/Guam'),
                    ('Pacific/Honolulu', 'Pacific/Honolulu'),
                    ('Pacific/Kiritimati', 'Pacific/Kiritimati'),
                    ('Pacific/Kosrae', 'Pacific/Kosrae'),
                    ('Pacific/Kwajalein', 'Pacific/Kwajalein'),
                    ('Pacific/Majuro', 'Pacific/Majuro'),
                    ('Pacific/Marquesas', 'Pacific/Marquesas'),
                    ('Pacific/Midway', 'Pacific/Midway'),
                    ('Pacific/Nauru', 'Pacific/Nauru'),
                    ('Pacific/Niue', 'Pacific/Niue'),
                    ('Pacific/Norfolk', 'Pacific/Norfolk'),
                    ('Pacific/Noumea', 'Pacific/Noumea'),
                    ('Pacific/Pago_Pago', 'Pacific/Pago_Pago'),
                    ('Pacific/Palau', 'Pacific/Palau'),
                    ('Pacific/Pitcairn', 'Pacific/Pitcairn'),
                    ('Pacific/Pohnpei', 'Pacific/Pohnpei'),
                    ('Pacific/Port_Moresby', 'Pacific/Port_Moresby'),
                    ('Pacific/Rarotonga', 'Pacific/Rarotonga'),
                    ('Pacific/Saipan', 'Pacific/Saipan'),
                    ('Pacific/Tahiti', 'Pacific/Tahiti'),
                    ('Pacific/Tarawa', 'Pacific/Tarawa'),
                    ('Pacific/Tongatapu', 'Pacific/Tongatapu'),
                    ('Pacific/Wake', 'Pacific/Wake'),
                    ('Pacific/Wallis', 'Pacific/Wallis'),
                    ('US/Alaska', 'US/Alaska'),
                    ('US/Arizona', 'US/Arizona'),
                    ('US/Central', 'US/Central'),
                    ('US/Eastern', 'US/Eastern'),
                    ('US/Hawaii', 'US/Hawaii'),
                    ('US/Mountain', 'US/Mountain'),
                    ('US/Pacific', 'US/Pacific'),
                    ('UTC', 'UTC')
                ], max_length=48, verbose_name='Timezone'
            ),
        ),
    ]



from django.apps import apps
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType

from mayan.apps.common.classes import QuerysetParametersSerializer
from mayan.celery import app

from .classes import ActionExporter
from .events import event_events_cleared
from .permissions import permission_events_clear


@app.task(ignore_result=True)
def task_event_queryset_clear(
    decomposed_queryset, target_content_type_id=None, target_object_id=None,
    user_id=None
):
    AccessControlList = apps.get_model(
        app_label='acls', model_name='AccessControlList'
    )

    queryset = QuerysetParametersSerializer.rebuild(
        decomposed_queryset=decomposed_queryset
    )

    if user_id:
        user = get_user_model().objects.get(pk=user_id)
    else:
        user = None

    if target_content_type_id:
        target_content_type = ContentType.objects.get(
            pk=target_content_type_id
        )
        target = target_content_type.get_object_for_this_type(
            pk=target_object_id
        )
    else:
        target = user

    AccessControlList = apps.get_model(
        app_label='acls', model_name='AccessControlList'
    )

    if user:
        queryset = AccessControlList.objects.restrict_queryset(
            queryset=queryset,
            permission=permission_events_clear,
            user=user
        )

    commit_event = queryset.exists()

    queryset.delete()

    if commit_event:
        event_events_cleared.commit(actor=user, target=target)


@app.task(ignore_result=True)
def task_event_queryset_export(decomposed_queryset, user_id=None):
    queryset = QuerysetParametersSerializer.rebuild(
        decomposed_queryset=decomposed_queryset
    )

    if user_id:
        user = get_user_model().objects.get(pk=user_id)
    else:
        user = None

    ActionExporter(queryset=queryset).export_to_download_file(user=user)



from django.contrib.auth import get_user_model


def get_user_queryset(user=None):
    if user and (user.is_superuser or user.is_staff):
        queryset = get_user_model().objects.all()
    else:
        queryset = get_user_model().objects.filter(
            is_superuser=False, is_staff=False
        )

    return queryset.order_by('username')



from django.contrib.admin.utils import help_text_for_field, label_for_field
from django.core.exceptions import FieldDoesNotExist


class AutoHelpTextLabelFieldMixin:
    def bind(self, *args, **kwargs):
        result = super().bind(*args, **kwargs)

        try:
            model = self.root.Meta.model
        except AttributeError:
            return result
        else:
            try:
                field = model._meta.get_field(field_name=self.source)
            except FieldDoesNotExist:
                return result
            else:
                field_name = field.name

                if not self.label:
                    self.label = label_for_field(
                        model=model, name=field_name
                    )

                if not self.help_text:
                    self.help_text = help_text_for_field(
                        model=model, name=field_name
                    )

                return result



from django.conf.urls import url

from .api_views import APISourceActionView, APISourceListView, APISourceView
from .views.document_file_views import DocumentFileUploadInteractiveView
from .views.document_views import DocumentUploadInteractiveView
from .views.source_views import (
    SourceActionView, SourceBackendSelectionView, SourceCreateView,
    SourceDeleteView, SourceEditView, SourceListView, SourceTestView
)
from .wizards import DocumentCreateWizard

urlpatterns = [
    # Document create views

    url(
        regex=r'^documents/create/from/local/multiple/$',
        name='document_create_multiple', view=DocumentCreateWizard.as_view()
    ),
    url(
        regex=r'^documents/upload/new/interactive/(?P<source_id>\d+)/$',
        name='document_upload_interactive',
        view=DocumentUploadInteractiveView.as_view()
    ),
    url(
        regex=r'^documents/upload/new/interactive/$',
        name='document_upload_interactive',
        view=DocumentUploadInteractiveView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/files/upload/interactive/(?P<source_id>\d+)/$',
        name='document_file_upload',
        view=DocumentFileUploadInteractiveView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/files/upload/interactive/$',
        name='document_file_upload',
        view=DocumentFileUploadInteractiveView.as_view()
    ),

    # Setup views

    url(
        regex=r'^sources/$', name='source_list',
        view=SourceListView.as_view()
    ),
    url(
        regex=r'^sources/backend/selection/$',
        name='source_backend_selection',
        view=SourceBackendSelectionView.as_view()
    ),
    url(
        regex=r'^sources/(?P<backend_path>[a-zA-Z0-9_.]+)/create/$',
        name='source_create', view=SourceCreateView.as_view()
    ),
    url(
        regex=r'^sources/(?P<source_id>\d+)/actions/(?P<action_name>[a-zA-Z0-9_.]+)/$',
        name='source_action', view=SourceActionView.as_view()
    ),
    url(
        regex=r'^sources/(?P<source_id>\d+)/delete/$',
        name='source_delete', view=SourceDeleteView.as_view()
    ),
    url(
        regex=r'^sources/(?P<source_id>\d+)/edit/$', name='source_edit',
        view=SourceEditView.as_view()
    ),
    url(
        regex=r'^sources/(?P<source_id>\d+)/test/$',
        name='source_test', view=SourceTestView.as_view()
    ),
]

api_urls = [
    url(
        regex=r'^sources/$', name='source-list',
        view=APISourceListView.as_view()
    ),
    url(
        regex=r'^sources/(?P<source_id>[0-9]+)/$',
        name='source-detail', view=APISourceView.as_view()
    ),
    url(
        regex=r'^sources/(?P<source_id>[0-9]+)/actions/(?P<action_name>[a-zA-Z0-9_.]+)/$',
        name='source-action', view=APISourceActionView.as_view()
    )
]



import shlex

from django.core.exceptions import ValidationError
from django.db import models
from django.urls import reverse
from django.utils.encoding import force_text
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.validators import YAMLValidator
from mayan.apps.common.serialization import yaml_load
from mayan.apps.databases.model_mixins import ExtraDataModelMixin
from mayan.apps.documents.models import Document, DocumentType
from mayan.apps.events.classes import EventManagerMethodAfter, EventManagerSave
from mayan.apps.events.decorators import method_event
from mayan.apps.templating.classes import Template

from .classes import MetadataLookup
from .events import (
    event_document_metadata_added, event_document_metadata_edited,
    event_document_metadata_removed, event_metadata_type_created,
    event_metadata_type_edited, event_metadata_type_relationship_updated
)
from .managers import DocumentTypeMetadataTypeManager, MetadataTypeManager


class MetadataType(ExtraDataModelMixin, models.Model):
    """
    Model to store a type of metadata. Metadata are user defined properties
    that can be assigned a value for each document. Metadata types need
    to be assigned to a document type before they can be used.
    """
    name = models.CharField(
        max_length=48,
        help_text=_(
            'Name used by other apps to reference this metadata type. '
            'Do not use python reserved words, or spaces.'
        ),
        unique=True, verbose_name=_('Name')
    )
    label = models.CharField(
        help_text=_('Short description of this metadata type.'),
        max_length=48, verbose_name=_('Label')
    )
    default = models.CharField(
        blank=True, max_length=128, null=True, help_text=_(
            'Enter a template to render.'
        ), verbose_name=_('Default')
    )
    lookup = models.TextField(
        blank=True, null=True, help_text=_(
            'Enter a template to render. Must result in a comma delimited '
            'string.'
        ), verbose_name=_('Lookup')
    )
    validation = models.CharField(
        blank=True, help_text=_(
            'The validator will reject data entry if the value entered does '
            'not conform to the expected format.'
        ), max_length=224, verbose_name=_('Validator')
    )
    validation_arguments = models.TextField(
        blank=True, help_text=_(
            'Enter the arguments for the validator in YAML format.'
        ), validators=[YAMLValidator()], verbose_name=_(
            'Validator arguments'
        )
    )
    parser = models.CharField(
        blank=True, help_text=_(
            'The parser will reformat the value entered to conform to the '
            'expected format.'
        ), max_length=224, verbose_name=_('Parser')
    )
    parser_arguments = models.TextField(
        blank=True, help_text=_(
            'Enter the arguments for the parser in YAML format.'
        ), validators=[YAMLValidator()], verbose_name=_(
            'Parser arguments'
        )
    )

    objects = MetadataTypeManager()

    class Meta:
        ordering = ('label',)
        verbose_name = _('Metadata type')
        verbose_name_plural = _('Metadata types')

    def __str__(self):
        return self.label

    @staticmethod
    def comma_splitter(string):
        splitter = shlex.shlex(string, posix=True)
        splitter.whitespace = ','
        splitter.whitespace_split = True
        splitter.commenters = ''
        return [force_text(s=e) for e in splitter]

    def get_absolute_url(self):
        return reverse(
            viewname='metadata:metadata_type_edit', kwargs={
                'metadata_type_id': self.pk
            }
        )

    def get_default_value(self):
        template = Template(template_string=self.default)
        return template.render()

    def get_lookup_values(self):
        template = Template(template_string=self.lookup)
        return MetadataType.comma_splitter(
            template.render(context=MetadataLookup.get_as_context())
        )

    def get_required_for(self, document_type):
        """
        Return a queryset of metadata types that are required for the
        specified document type.
        """
        return document_type.metadata.filter(
            required=True, metadata_type=self
        ).exists()

    def natural_key(self):
        return (self.name,)

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'event': event_metadata_type_created,
            'target': 'self',
        },
        edited={
            'event': event_metadata_type_edited,
            'target': 'self',
        }
    )
    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)

    def validate_value(self, document_type, value):
        # Check default
        if not value and self.default:
            value = self.get_default_value()

        if not value and self.get_required_for(document_type=document_type):
            raise ValidationError(
                _('"%s" is required for this document type.') % self.label
            )

        if self.lookup:
            lookup_options = self.get_lookup_values()

            if value and value not in lookup_options:
                raise ValidationError(
                    _('Value is not one of the provided options.')
                )

        if self.validation:
            validator_class = import_string(dotted_path=self.validation)
            validator_arguments = yaml_load(
                stream=self.validation_arguments or '{}'
            )
            validator = validator_class(**validator_arguments)
            try:
                validator.validate(value)
            except ValidationError as exception:
                raise ValidationError(
                    _(
                        'Metadata type validation error; %(exception)s'
                    ) % {'exception': ','.join(exception)}
                ) from exception

        if self.parser:
            parser_class = import_string(dotted_path=self.parser)
            parser_arguments = yaml_load(
                stream=self.parser_arguments or '{}'
            )
            parser = parser_class(**parser_arguments)
            value = parser.parse(value)

        return value


class DocumentMetadata(ExtraDataModelMixin, models.Model):
    """
    Model used to link an instance of a metadata type with a value to a
    document.
    """
    document = models.ForeignKey(
        on_delete=models.CASCADE, related_name='metadata', to=Document,
        verbose_name=_('Document')
    )
    metadata_type = models.ForeignKey(
        on_delete=models.CASCADE, to=MetadataType, verbose_name=_('Type')
    )
    value = models.CharField(
        blank=True, db_index=True, help_text=_(
            'The actual value stored in the metadata type field for '
            'the document.'
        ), max_length=255, null=True, verbose_name=_('Value')
    )

    class Meta:
        ordering = ('metadata_type',)
        unique_together = ('document', 'metadata_type')
        verbose_name = _('Document metadata')
        verbose_name_plural = _('Document metadata')

    def __str__(self):
        return force_text(s=self.metadata_type)

    def clean_fields(self, *args, **kwargs):
        super().clean_fields(*args, **kwargs)

        self.value = self.metadata_type.validate_value(
            document_type=self.document.document_type, value=self.value
        )

    @method_event(
        event_manager_class=EventManagerMethodAfter,
        action_object='metadata_type',
        event=event_document_metadata_removed,
        target='document'
    )
    def delete(self, enforce_required=True, *args, **kwargs):
        """
        Delete a metadata from a document. enforce_required which defaults
        to True, prevents deletion of required metadata at the model level.
        It used set to False when deleting document metadata on document
        type change.
        """
        if enforce_required and self.document.document_type.metadata.filter(required=True).filter(metadata_type=self.metadata_type).exists():
            raise ValidationError(
                _('Metadata type is required for this document type.')
            )

        return super().delete(*args, **kwargs)

    def natural_key(self):
        return self.document.natural_key() + self.metadata_type.natural_key()
    natural_key.dependencies = ['documents.Document', 'metadata.MetadataType']

    @property
    def is_required(self):
        """
        Return a boolean value of True of this metadata instance's parent type
        is required for the stored document type.
        """
        return self.metadata_type.get_required_for(
            document_type=self.document.document_type
        )

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'action_object': 'metadata_type',
            'event': event_document_metadata_added,
            'target': 'document'
        },
        edited={
            'action_object': 'metadata_type',
            'event': event_document_metadata_edited,
            'target': 'document'
        }
    )
    def save(self, *args, **kwargs):
        if not self.document.document_type.metadata.filter(metadata_type=self.metadata_type).exists():
            raise ValidationError(
                _('Metadata type is not valid for this document type.')
            )

        return super().save(*args, **kwargs)


class DocumentTypeMetadataType(ExtraDataModelMixin, models.Model):
    """
    Model used to store the relationship between a metadata type and a
    document type.
    """
    document_type = models.ForeignKey(
        on_delete=models.CASCADE, related_name='metadata', to=DocumentType,
        verbose_name=_('Document type')
    )
    metadata_type = models.ForeignKey(
        on_delete=models.CASCADE, related_name='document_types', to=MetadataType,
        verbose_name=_('Metadata type')
    )
    required = models.BooleanField(default=False, verbose_name=_('Required'))

    objects = DocumentTypeMetadataTypeManager()

    class Meta:
        ordering = ('metadata_type',)
        unique_together = ('document_type', 'metadata_type')
        verbose_name = _('Document type metadata type options')
        verbose_name_plural = _('Document type metadata types options')

    def __str__(self):
        return force_text(s=self.metadata_type)

    @method_event(
        event_manager_class=EventManagerMethodAfter,
        action_object='metadata_type',
        event=event_metadata_type_relationship_updated,
        target='document_type'
    )
    def delete(self, *args, **kwargs):
        return super().delete(*args, **kwargs)

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'action_object': 'metadata_type',
            'event': event_metadata_type_relationship_updated,
            'target': 'document_type'
        },
        edited={
            'action_object': 'metadata_type',
            'event': event_metadata_type_relationship_updated,
            'target': 'document_type'
        }
    )
    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)



from django.db import connection
from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.search import (
    document_file_search, document_file_page_search, document_search,
    document_version_search, document_version_page_search
)
from mayan.apps.dynamic_search.classes import SearchModel

from .permissions import permission_tag_view


def transformation_format_uuid(term_string):
    if connection.vendor in ('mysql', 'sqlite'):
        return term_string.replace('-', '')
    else:
        return term_string


# Document

document_search.add_model_field(field='tags__label', label=_('Tag label'))
document_search.add_model_field(field='tags__color', label=_('Tag color'))

# Document file

document_file_search.add_model_field(
    field='document__tags__label', label=_('Document tags')
)

# Document file page

document_file_page_search.add_model_field(
    field='document_file__document__tags__label', label=_('Document tags')
)

# Document version

document_version_search.add_model_field(
    field='document__tags__label', label=_('Document tags')
)

# Document version page

document_version_page_search.add_model_field(
    field='document_version__document__tags__label', label=_('Document tags')
)

# Tag

tag_search = SearchModel(
    app_label='tags', model_name='Tag',
    permission=permission_tag_view,
    serializer_path='mayan.apps.tags.serializers.TagSerializer'
)
tag_search.add_model_field(field='label')
tag_search.add_model_field(field='color')

tag_search.add_model_field(
    field='documents__document_type__label', label=_('Document type')
)
tag_search.add_model_field(
    field='documents__label', label=_('Document label')
)
tag_search.add_model_field(
    field='documents__description', label=_('Document description')
)
tag_search.add_model_field(
    field='documents__uuid', label=_('Document UUID'),
    transformation_function=transformation_format_uuid
)

tag_search.add_model_field(
    field='documents__files__checksum', label=_('Document file checksum')
)
tag_search.add_model_field(
    field='documents__files__mimetype', label=_('Document file MIME type')
)



from django import forms
from django.core.exceptions import ValidationError
from django.forms.formsets import formset_factory
from django.utils.text import format_lazy
from django.utils.translation import ugettext_lazy as _

from mayan.apps.templating.fields import TemplateField
from mayan.apps.views.forms import RelationshipForm

from .classes import MetadataLookup, MetadataParser, MetadataValidator
from .models import MetadataType


class DocumentMetadataForm(forms.Form):
    metadata_type_id = forms.CharField(label=_('ID'), widget=forms.HiddenInput)
    metadata_type_name = forms.CharField(
        label=_('Name'), required=False,
        widget=forms.TextInput(
            attrs={
                'disabled': 'disabled',
                'readonly': 'readonly'
            }
        )
    )
    value = forms.CharField(
        label=_('Value'), required=False, widget=forms.TextInput(
            attrs={'class': 'metadata-value'}
        )
    )
    update = forms.BooleanField(
        initial=True, label=_('Update'), required=False
    )

    class Media:
        js = ('metadata/js/metadata_form.js',)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Set form fields initial values
        if 'initial' in kwargs:
            self.metadata_type = kwargs['initial']['metadata_type']
            self.document_type = kwargs['initial']['document_type']
            required_string = ''

            required = self.metadata_type.get_required_for(
                document_type=self.document_type
            )

            if required:
                self.fields['value'].required = True
                required_string = ' (%s)' % _('Required')
            else:
                self.fields['value'].required = False
                self.fields['update'].initial = False

            self.fields['metadata_type_name'].initial = '%s%s' % (
                (
                    self.metadata_type.label if self.metadata_type.label else self.metadata_type.name
                ),
                required_string
            )
            self.fields['metadata_type_id'].initial = self.metadata_type.pk

            if self.metadata_type.lookup:
                try:
                    self.fields['value'] = forms.ChoiceField(
                        label=self.fields['value'].label
                    )
                    choices = self.metadata_type.get_lookup_values()
                    choices = list(zip(choices, choices))
                    if not required:
                        choices.insert(0, ('', '------'))
                    self.fields['value'].choices = choices
                    self.fields['value'].required = required
                    self.fields['value'].widget.attrs.update(
                        {'class': 'metadata-value'}
                    )
                except Exception as exception:
                    self.fields['value'].initial = _(
                        'Lookup value error: %s'
                    ) % exception
                    self.fields['value'].widget = forms.TextInput(
                        attrs={'readonly': 'readonly'}
                    )

            if self.metadata_type.default:
                try:
                    self.fields[
                        'value'
                    ].initial = self.metadata_type.get_default_value()
                except Exception as exception:
                    self.fields['value'].initial = _(
                        'Default value error: %s'
                    ) % exception
                    self.fields['value'].widget = forms.TextInput(
                        attrs={'readonly': 'readonly'}
                    )

    def clean(self):
        metadata_type = getattr(self, 'metadata_type', None)

        if metadata_type:
            required = self.metadata_type.get_required_for(
                document_type=self.document_type
            )

            # Enforce required only if the metadata has no previous value.
            if required and not self.cleaned_data.get('value') and not self.cleaned_data.get('update'):
                raise ValidationError(
                    _(
                        '"%s" is required for this document type.'
                    ) % self.metadata_type.label
                )

        if self.cleaned_data.get('update') and hasattr(self, 'metadata_type'):
            self.cleaned_data['value'] = self.metadata_type.validate_value(
                document_type=self.document_type,
                value=self.cleaned_data.get('value')
            )

        return self.cleaned_data


DocumentMetadataFormSet = formset_factory(form=DocumentMetadataForm, extra=0)


class DocumentMetadataAddForm(forms.Form):
    metadata_type = forms.ModelMultipleChoiceField(
        help_text=_('Metadata types to be added to the selected documents.'),
        label=_('Metadata type'), queryset=MetadataType.objects.all(),
        widget=forms.SelectMultiple(
            attrs={'class': 'select2'},
        )
    )

    def __init__(self, *args, **kwargs):
        document_type = kwargs.pop('document_type', None)

        if document_type:
            queryset = kwargs.pop(
                'queryset', MetadataType.objects.get_for_document_type(
                    document_type=document_type
                )
            )
        else:
            queryset = MetadataType.objects.none()

        super().__init__(*args, **kwargs)

        self.fields['metadata_type'].queryset = queryset


class DocumentMetadataRemoveForm(DocumentMetadataForm):
    update = forms.BooleanField(
        initial=False, label=_('Remove'), required=False
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields.pop('value')

    def clean(self):
        return super(forms.Form, self).clean()


DocumentMetadataRemoveFormSet = formset_factory(
    form=DocumentMetadataRemoveForm, extra=0
)


class MetadataTypeForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['default'] = TemplateField(
            initial_help_text=self.fields['default'].help_text, required=False
        )
        self.fields['lookup'] = TemplateField(
            initial_help_text=format_lazy(
                '{}{}{}',
                self.fields['lookup'].help_text,
                _(' Available template context variables: '),
                MetadataLookup.get_as_help_text()
            ), required=False
        )
        self.fields['parser'].widget = forms.widgets.Select(
            choices=MetadataParser.get_choices(add_blank=True)
        )
        self.fields['validation'].widget = forms.widgets.Select(
            choices=MetadataValidator.get_choices(add_blank=True)
        )

    class Meta:
        fields = (
            'name', 'label', 'default', 'lookup', 'validation',
            'validation_arguments', 'parser', 'parser_arguments'
        )
        model = MetadataType


class DocumentTypeMetadataTypeRelationshipForm(RelationshipForm):
    RELATIONSHIP_TYPE_NONE = 'none'
    RELATIONSHIP_TYPE_OPTIONAL = 'optional'
    RELATIONSHIP_TYPE_REQUIRED = 'required'
    RELATIONSHIP_CHOICES = (
        (RELATIONSHIP_TYPE_NONE, _('None')),
        (RELATIONSHIP_TYPE_OPTIONAL, _('Optional')),
        (RELATIONSHIP_TYPE_REQUIRED, _('Required')),
    )

    def get_relationship_type(self):
        relationship_queryset = self.get_relationship_queryset()

        if relationship_queryset.exists():
            if relationship_queryset.get().required:
                return self.RELATIONSHIP_TYPE_REQUIRED
            else:
                return self.RELATIONSHIP_TYPE_OPTIONAL
        else:
            return self.RELATIONSHIP_TYPE_NONE

    def save_relationship_none(self):
        instance = self.get_relationship_instance()
        instance._event_actor = self._event_actor
        instance.delete()

    def save_relationship_optional(self):
        instance = self.get_relationship_instance()
        instance.required = False
        instance._event_actor = self._event_actor
        instance.save()

    def save_relationship_required(self):
        instance = self.get_relationship_instance()
        instance.required = True
        instance._event_actor = self._event_actor
        instance.save()


DocumentTypeMetadataTypeRelationshipFormSetBase = formset_factory(
    form=DocumentTypeMetadataTypeRelationshipForm, extra=0
)


class DocumentTypeMetadataTypeRelationshipFormSet(DocumentTypeMetadataTypeRelationshipFormSetBase):
    def __init__(self, *args, **kwargs):
        _event_actor = kwargs.pop('_event_actor')
        super().__init__(*args, **kwargs)
        self.form_kwargs.update({'_event_actor': _event_actor})



from furl import furl

from django.conf import settings
from django.contrib.auth import authenticate
from django.test import override_settings
from django.urls import reverse
from django.utils.http import urlunquote_plus

from mayan.apps.authentication.classes import AuthenticationBackend
from mayan.apps.authentication.tests.mixins import LoginViewTestMixin
from mayan.apps.smart_settings.classes import SettingNamespace
from mayan.apps.testing.tests.base import GenericViewTestCase

from .literals import (
    PATH_AUTHENTICATION_BACKEND_EMAIL_OTP,
    PATH_AUTHENTICATION_BACKEND_USERNAME_OTP
)
from .mixins import AuthenticationOTPTestMixin


class AuthenticationOTPBackendTestCase(
    AuthenticationOTPTestMixin, LoginViewTestMixin, GenericViewTestCase
):
    authenticated_url = reverse(viewname='common:home')
    authentication_url = urlunquote_plus(
        furl(
            path=reverse(settings.LOGIN_URL), args={
                'next': authenticated_url
            }
        ).tostr()
    )
    auto_login_user = False
    create_test_case_superuser = True

    def setUp(self):
        super().setUp()
        SettingNamespace.invalidate_cache_all()

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_EMAIL_OTP)
    def test_authentication_backend_email_no_otp(self):
        AuthenticationBackend.cls_initialize()

        user = authenticate(
            username=self._test_case_superuser.email,
            password=self._test_case_superuser.cleartext_password
        )
        self.assertEqual(user, self._test_case_superuser)

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_EMAIL_OTP)
    def test_authentication_backend_email_with_otp(self):
        AuthenticationBackend.cls_initialize()

        self._enable_test_otp()

        user = authenticate(
            username=self._test_case_superuser.email,
            password=self._test_case_superuser.cleartext_password,
        )
        self.assertEqual(user, self._test_case_superuser)

        user = authenticate(
            factor_name='otp_token', otp_token=self._test_token,
            user=user
        )
        self.assertEqual(user, self._test_case_superuser)

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_USERNAME_OTP)
    def test_authentication_backend_username_no_otp(self):
        AuthenticationBackend.cls_initialize()

        user = authenticate(
            username=self._test_case_superuser.username,
            password=self._test_case_superuser.cleartext_password
        )
        self.assertEqual(user, self._test_case_superuser)

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_USERNAME_OTP)
    def test_authentication_backend_username_with_otp(self):
        AuthenticationBackend.cls_initialize()

        self._enable_test_otp()

        user = authenticate(
            username=self._test_case_superuser.username,
            password=self._test_case_superuser.cleartext_password
        )
        self.assertEqual(user, self._test_case_superuser)

        user = authenticate(
            factor_name='otp_token', otp_token=self._test_token,
            user=user
        )
        self.assertEqual(user, self._test_case_superuser)



from django.core.management.base import BaseCommand

from ...classes import SearchBackend


class Command(BaseCommand):
    help = 'Show search backend statistics.'

    def handle(self, *args, **options):
        backend = SearchBackend.get_instance()

        result = backend.get_status()

        self.stdout.write(msg=result)



from django.apps import apps
from django.utils.translation import ugettext_lazy as _


def handler_create_default_document_source(sender, **kwargs):
    from .source_backends.literals import SOURCE_UNCOMPRESS_CHOICE_ASK
    from .source_backends.web_form_backends import SourceBackendWebForm

    Source = apps.get_model(
        app_label='sources', model_name='Source'
    )

    if not Source.objects.filter(backend_path=SourceBackendWebForm.get_class_path()).count():
        Source.objects.create_backend(
            label=_('Default'),
            backend_path=SourceBackendWebForm.get_class_path(),
            backend_data={
                'uncompress': SOURCE_UNCOMPRESS_CHOICE_ASK
            }
        )


def handler_delete_interval_source_periodic_task(sender, instance, **kwargs):
    Source = apps.get_model(
        app_label='sources', model_name='Source'
    )

    for source in Source.objects.all():
        backend_instance = source.get_backend_instance()

        if backend_instance.kwargs.get('document_type') == instance:
            try:
                backend_instance.delete_periodic_task()
            except AttributeError:
                """
                The source has a document type but is not a periodic source,
                """


def handler_initialize_periodic_tasks(sender, **kwargs):
    Source = apps.get_model(
        app_label='sources', model_name='Source'
    )

    for source in Source.objects.filter(enabled=True):
        backend_instance = source.get_backend_instance()
        backend_instance.save()



from mayan.apps.converter.settings import setting_image_generation_timeout

from .events import event_ocr_document_version_submitted
from .literals import TASK_DOCUMENT_VERSION_PAGE_OCR_TIMEOUT
from .tasks import task_document_version_ocr_process


def method_document_ocr_submit(self, _user=None):
    version_active = self.version_active
    # Don't error out if document has no version
    if version_active:
        version_active.submit_for_ocr(_user=_user)


def method_document_version_ocr_submit(self, _user=None):
    event_ocr_document_version_submitted.commit(
        action_object=self.document, actor=_user, target=self
    )

    if _user:
        user_id = _user.pk
    else:
        user_id = None

    # Timeout calculation logic:
    # Total timeout for the document version task should be the
    # file page render timeout + the version page timeout + version page OCR
    # timeout x the total version pages. This decreases the probability of
    # the OCR task getting killed before the document file and version page
    # rendering finishes.

    task_document_version_ocr_process.apply_async(
        kwargs={
            'document_version_id': self.pk, 'user_id': user_id
        }, timeout=(
            TASK_DOCUMENT_VERSION_PAGE_OCR_TIMEOUT + setting_image_generation_timeout.value * 2
        ) * self.pages.count()
    )



from functools import reduce
import logging
import operator

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied
from django.db import models
from django.db.models import CharField, Q, Value
from django.db.models.functions import Cast, Concat
from django.utils.encoding import force_text
from django.utils.translation import ugettext

from mayan.apps.common.utils import (
    get_related_field, resolve_attribute, return_related
)
from mayan.apps.permissions import Permission
from mayan.apps.permissions.models import StoredPermission

from .exceptions import PermissionNotValidForClass
from .classes import ModelPermission

logger = logging.getLogger(name=__name__)


class AccessControlListManager(models.Manager):
    """
    Implement a 3 tier permission system, involving a permissions, an actor
    and an object
    """
    def _get_acl_filters(
        self, queryset, stored_permission, user, fk_field_cast=None,
        related_field_name=None
    ):
        """
        This method does the bulk of the work. It generates filters for the
        AccessControlList model to determine if there are ACL entries for the
        members of the queryset's model provided.
        """
        # Determine which of the cases we need to address:
        # 1: No related field
        # 2: Related field
        # 3: Related field that is Generic Foreign Key
        # 4: No related field, but has an inherited related field, solved by
        # recursion, branches to #2 or #3.
        # 5: Inherited field of a related field
        # 6: Inherited field of a related field that is Generic Foreign Key
        # -- Not addressed yet --
        # 7: Has a related function
        result = []

        if related_field_name:
            related_field = get_related_field(
                model=queryset.model, related_field_name=related_field_name
            )

            if isinstance(related_field, GenericForeignKey):
                # Case 3: Generic Foreign Key, multiple ContentTypes + object
                # id combinations.
                # Also handles case #6 using the parent related field
                # reference template.

                # Craft a double underscore reference to a previous related
                # field in the case where multiple related fields are
                # associated.
                # Example: object_layer__content_type
                recursive_related_reference = '__'.join(related_field_name.split('__')[0:-1])

                # If there is at least one parent related field we add a
                # double underscore to make it a valid filter template.
                if recursive_related_reference:
                    recursive_related_reference = '{}__'.format(recursive_related_reference)

                content_type_object_id_queryset = queryset.annotate(
                    ct_fk_combination=Concat(
                        '{}{}'.format(
                            recursive_related_reference, related_field.ct_field
                        ), Value('-'),
                        '{}{}'.format(
                            recursive_related_reference, related_field.fk_field
                        ), output_field=CharField()
                    )
                ).values('ct_fk_combination')

                acl_filter = self.annotate(
                    ct_fk_combination=Concat(
                        'content_type', Value('-'), 'object_id',
                        output_field=CharField()
                    )
                ).filter(
                    permissions=stored_permission, role__groups__user=user,
                    ct_fk_combination__in=content_type_object_id_queryset
                )

                if fk_field_cast:
                    clean_acl_filter = acl_filter.annotate(
                        clean_object_id=Cast(
                            'object_id', output_field=fk_field_cast()
                        )
                    ).values_list('clean_object_id')
                else:
                    clean_acl_filter = acl_filter.values('object_id')

                field_lookup = '{}{}__in'.format(
                    recursive_related_reference, related_field.fk_field
                )
                result.append(Q(**{field_lookup: clean_acl_filter}))
            else:
                # Case 2: Related field of a single type, single ContentType,
                # multiple object id.
                content_type = ContentType.objects.get_for_model(
                    model=related_field.related_model
                )
                field_lookup = '{}_id__in'.format(related_field_name)
                acl_filter = self.filter(
                    content_type=content_type, permissions=stored_permission,
                    role__groups__user=user
                ).values('object_id')
                # Don't add empty filters otherwise the default AND operator
                # of the Q object will return an empty queryset when reduced
                # and filter out objects that should be in the final queryset.
                if acl_filter:
                    result.append(Q(**{field_lookup: acl_filter}))

                # Case 5: Related field, has an inherited related field itself
                # Bubble up permission check.
                # Recurse and reduce.
                try:
                    related_field_model_inheritances = (
                        ModelPermission.get_inheritances(
                            model=related_field.related_model
                        )
                    )
                except KeyError:
                    pass
                else:
                    relation_result = []
                    for related_field_model_inheritance in related_field_model_inheritances:
                        new_related_field_name = '{}__{}'.format(related_field_name, related_field_model_inheritance['field_name'])
                        related_field_inherited_acl_queries = self._get_acl_filters(
                            fk_field_cast=related_field_model_inheritance['fk_field_cast'],
                            queryset=queryset,
                            stored_permission=stored_permission, user=user,
                            related_field_name=new_related_field_name
                        )
                        if related_field_inherited_acl_queries:
                            relation_result.append(
                                reduce(
                                    operator.and_,
                                    related_field_inherited_acl_queries
                                )
                            )

                    if relation_result:
                        result.append(reduce(operator.or_, relation_result))
        else:
            # Case 1: Original model, single ContentType, multiple object id.
            content_type = ContentType.objects.get_for_model(
                model=queryset.model
            )
            field_lookup = 'id__in'
            acl_filter = self.filter(
                content_type=content_type, permissions=stored_permission,
                role__groups__user=user
            ).values('object_id')
            result.append(Q(**{field_lookup: acl_filter}))

            # Case 4: Original model, has an inherited related field.
            try:
                inheritances = (
                    ModelPermission.get_inheritances(
                        model=queryset.model
                    )
                )
            except KeyError:
                """
                Does not have inheritance entries. Proceed to next case.
                """
            else:
                relation_result = []

                for inheritance in inheritances:
                    inherited_acl_queries = self._get_acl_filters(
                        fk_field_cast=inheritance['fk_field_cast'],
                        queryset=queryset, stored_permission=stored_permission,
                        related_field_name=inheritance['field_name'], user=user
                    )
                    if inherited_acl_queries:
                        relation_result.append(
                            reduce(operator.and_, inherited_acl_queries)
                        )

                if relation_result:
                    result.append(reduce(operator.or_, relation_result))

            # Case 7: Has a function.
            try:
                field_query_function = ModelPermission.get_field_query_function(
                    model=queryset.model
                )
            except KeyError:
                pass
            else:
                function_results = field_query_function()

                # Filter by the model's content type.
                content_type = ContentType.objects.get_for_model(
                    model=queryset.model
                )
                acl_filter = self.filter(
                    content_type=content_type, permissions=stored_permission,
                    role__groups__user=user
                ).values('object_id')

                # Obtain a queryset of filtered, authorized model instances.
                acl_queryset = queryset.model._meta.default_manager.filter(
                    id__in=acl_filter
                ).filter(**function_results['acl_filter'])

                if 'acl_values' in function_results:
                    acl_queryset = acl_queryset.values(
                        *function_results['acl_values']
                    )

                # Get the final query using the filtered queryset as the
                # reference.
                result.append(
                    Q(**{function_results['field_lookup']: acl_queryset})
                )

        return result

    def check_access(self, obj, permissions, user):
        # Allow specific managers for models that have more than one
        # for example the Document model when checking for access for a trashed
        # document.

        meta = getattr(obj, '_meta', None)

        if not meta:
            logger.debug(
                ugettext(
                    'Object "%s" is not a model and cannot be checked for '
                    'access.'
                ) % force_text(s=obj)
            )
            return True
        else:
            manager = ModelPermission.get_manager(model=obj._meta.model)
            source_queryset = manager.all()

        restricted_queryset = manager.none()
        for permission in permissions:
            # Default relationship betweens permissions is OR.
            restricted_queryset = restricted_queryset | self.restrict_queryset(
                permission=permission, queryset=source_queryset, user=user
            )

        if restricted_queryset.filter(pk=obj.pk).exists():
            return True
        else:
            raise PermissionDenied(
                ugettext(message='Insufficient access for: %s') % force_text(
                    s=obj
                )
            )

    def restrict_queryset(self, permission, queryset, user):
        if not user.is_authenticated:
            return queryset.none()

        # Check directly granted permission via a role
        try:
            Permission.check_user_permissions(
                permissions=(permission,), user=user
            )
        except PermissionDenied:
            acl_filters = self._get_acl_filters(
                queryset=queryset,
                stored_permission=permission.stored_permission, user=user
            )

            final_query = None
            for acl_filter in acl_filters:
                if final_query is None:
                    final_query = acl_filter
                else:
                    final_query = final_query | acl_filter

            return queryset.filter(final_query)
        else:
            # User has direct permission assignment via a role, is superuser
            # or is staff. Return the entire queryset.
            return queryset

    def get_inherited_permissions(self, obj, role):
        # Get permission inherited from a related object's ACLs.
        queryset = self._get_inherited_object_permissions(obj=obj, role=role)

        # Get permission granted to the role
        queryset = queryset | role.permissions.all()

        # Filter the permissions to the ones that apply to the model.
        queryset = ModelPermission.get_for_instance(
            instance=obj
        ).filter(
            pk__in=queryset
        )

        return queryset

    def _get_inherited_object_permissions(self, obj, role):
        queryset = StoredPermission.objects.none()

        if not obj:
            return queryset

        try:
            inheritances = ModelPermission.get_inheritances(
                model=type(obj)
            )
        except KeyError:
            """
            Does not have inheritance to other models.
            """
        else:
            for inheritance in inheritances:
                try:
                    parent_object = resolve_attribute(
                        obj=obj, attribute=inheritance['field_name']
                    )
                except AttributeError:
                    # Parent accessor is not an attribute, try it as a related
                    # field.
                    parent_object = return_related(
                        instance=obj, related_field=inheritance['field_name']
                    )
                content_type = ContentType.objects.get_for_model(
                    model=parent_object
                )
                try:
                    queryset = queryset | self.get(
                        content_type=content_type, object_id=parent_object.pk,
                        role=role
                    ).permissions.all()
                except self.model.DoesNotExist:
                    pass

                if type(parent_object) == type(obj):
                    # Object and parent are of the same type. Break recursion.
                    return queryset
                else:
                    queryset = queryset | self._get_inherited_object_permissions(
                        obj=parent_object, role=role
                    )

        return queryset

    def grant(self, permission, role, obj):
        class_permissions = ModelPermission.get_for_class(klass=obj.__class__)
        if permission not in class_permissions:
            raise PermissionNotValidForClass

        content_type = ContentType.objects.get_for_model(model=obj)
        acl, created = self.get_or_create(
            content_type=content_type, object_id=obj.pk,
            role=role
        )

        acl.permissions.add(permission.stored_permission)

    def revoke(self, permission, role, obj):
        content_type = ContentType.objects.get_for_model(model=obj)
        acl, created = self.get_or_create(
            content_type=content_type, object_id=obj.pk,
            role=role
        )

        acl.permissions.remove(permission.stored_permission)

        if acl.permissions.count() == 0:
            acl.delete()



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('metadata', '0008_auto_20180310_1715'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='documenttypemetadatatype',
            options={
                'ordering': ('metadata_type',),
                'verbose_name': 'Document type metadata type options',
                'verbose_name_plural': 'Document type metadata types options'
            },
        ),
    ]



from contextlib import contextmanager
import sys

from django.utils.encoding import force_text


class NullFile:
    def write(self, string):
        """Writes here go nowhere"""


def as_id_list(items):
    return ','.join(
        [force_text(s=item.pk) for item in items]
    )


@contextmanager
def mute_stdout():
    stdout_old = sys.stdout
    sys.stdout = NullFile()
    yield
    sys.stdout = stdout_old



from mayan.apps.documents.events import (
    event_document_created, event_document_file_created,
    event_document_file_edited, event_document_version_created,
    event_document_version_page_created
)
from mayan.apps.documents.models import Document
from mayan.apps.documents.permissions import (
    permission_document_create, permission_document_file_new
)
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from .mixins.base_mixins import (
    DocumentFileUploadViewTestMixin, DocumentUploadWizardViewTestMixin
)
from .mixins.sane_scanner_source_mixins import SANEScannerSourceTestMixin


class SANEScannerDocumentUploadWizardViewTestCase(
    SANEScannerSourceTestMixin, DocumentUploadWizardViewTestMixin,
    GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def test_upload_interactive_view_no_permission(self):
        test_document_count = Document.objects.count()

        self._clear_events()

        response = self._request_upload_interactive_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Document.objects.count(), test_document_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_upload_interactive_view_with_document_type_access(self):
        self.grant_access(
            obj=self.test_document_type, permission=permission_document_create
        )

        test_document_count = Document.objects.count()

        self._clear_events()

        response = self._request_upload_interactive_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Document.objects.count(), test_document_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_upload_interactive_view_with_source_access(self):
        self.grant_access(
            obj=self.test_source, permission=permission_document_create
        )

        test_document_count = Document.objects.count()

        self._clear_events()

        response = self._request_upload_interactive_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Document.objects.count(), test_document_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_upload_interactive_view_with_full_access(self):
        self.grant_access(
            obj=self.test_document_type, permission=permission_document_create
        )
        self.grant_access(
            obj=self.test_source, permission=permission_document_create
        )

        test_document_count = Document.objects.count()

        self._clear_events()

        response = self._request_upload_interactive_view()
        self.assertContains(
            response=response, text=self.test_source.label, status_code=200
        )

        self.assertEqual(Document.objects.count(), test_document_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_upload_wizard_no_permission(self):
        test_document_count = Document.objects.count()

        self._clear_events()

        response = self._request_upload_wizard_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Document.objects.count(), test_document_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_upload_wizard_with_document_type_access(self):
        self.grant_access(
            obj=self.test_document_type, permission=permission_document_create
        )

        test_document_count = Document.objects.count()

        self._clear_events()

        response = self._request_upload_wizard_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Document.objects.count(), test_document_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_upload_wizard_with_source_access(self):
        self.grant_access(
            obj=self.test_source, permission=permission_document_create
        )

        test_document_count = Document.objects.count()

        self._clear_events()

        response = self._request_upload_wizard_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Document.objects.count(), test_document_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_upload_wizard_with_full_access(self):
        self.grant_access(
            obj=self.test_document_type, permission=permission_document_create
        )
        self.grant_access(
            obj=self.test_source, permission=permission_document_create
        )

        test_document_count = Document.objects.count()

        self._clear_events()

        response = self._request_upload_wizard_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Document.objects.count(), test_document_count + 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self.test_document_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, self._test_case_user)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )


class SANEScannerDocumentFileUploadViewTestCase(
    DocumentFileUploadViewTestMixin, SANEScannerSourceTestMixin,
    GenericDocumentViewTestCase
):
    def test_document_file_upload_view_no_permission(self):
        file_count = self.test_document.files.count()

        self._clear_events()

        response = self._request_document_file_upload_view()

        self.assertEqual(response.status_code, 404)

        self.test_document.refresh_from_db()
        self.assertEqual(self.test_document.files.count(), file_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_upload_view_with_document_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_file_new
        )
        file_count = self.test_document.files.count()

        self._clear_events()

        response = self._request_document_file_upload_view()
        self.assertEqual(response.status_code, 404)

        self.test_document.refresh_from_db()
        self.assertEqual(self.test_document.files.count(), file_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_upload_view_with_source_access(self):
        self.grant_access(
            obj=self.test_source, permission=permission_document_file_new
        )
        file_count = self.test_document.files.count()

        self._clear_events()

        response = self._request_document_file_upload_view()
        self.assertEqual(response.status_code, 404)

        self.test_document.refresh_from_db()
        self.assertEqual(self.test_document.files.count(), file_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_upload_view_with_full_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_file_new
        )
        self.grant_access(
            obj=self.test_source, permission=permission_document_file_new
        )
        file_count = self.test_document.files.count()

        self._clear_events()

        response = self._request_document_file_upload_view()
        self.assertEqual(response.status_code, 302)

        self.test_document.refresh_from_db()
        self.assertEqual(
            self.test_document.files.count(), file_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 4)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, test_document_file)
        self.assertEqual(events[0].verb, event_document_file_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_edited.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(events[2].target, test_document_version)
        self.assertEqual(events[2].verb, event_document_version_created.id)

        self.assertEqual(events[3].action_object, test_document_version)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, test_document_version_page)
        self.assertEqual(
            events[3].verb, event_document_version_page_created.id
        )

    def test_trashed_document_file_upload_view_with_full_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_file_new
        )
        self.grant_access(
            obj=self.test_source, permission=permission_document_file_new
        )
        file_count = self.test_document.files.count()

        self.test_document.delete()

        self._clear_events()

        response = self._request_document_file_upload_view()
        self.assertEqual(response.status_code, 404)

        self.test_document.refresh_from_db()
        self.assertEqual(
            self.test_document.files.count(), file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_upload_no_source_view_no_permission(self):
        file_count = self.test_document.files.count()

        self._clear_events()

        response = self._request_document_file_upload_no_source_view()
        self.assertEqual(response.status_code, 404)

        self.test_document.refresh_from_db()
        self.assertEqual(self.test_document.files.count(), file_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_upload_no_source_view_with_document_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_file_new
        )
        file_count = self.test_document.files.count()

        self._clear_events()

        response = self._request_document_file_upload_no_source_view()
        self.assertEqual(response.status_code, 404)

        self.test_document.refresh_from_db()
        self.assertEqual(self.test_document.files.count(), file_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_upload_no_source_view_with_source_access(self):
        self.grant_access(
            obj=self.test_source, permission=permission_document_file_new
        )
        file_count = self.test_document.files.count()

        self._clear_events()

        response = self._request_document_file_upload_no_source_view()
        self.assertEqual(response.status_code, 404)

        self.test_document.refresh_from_db()
        self.assertEqual(self.test_document.files.count(), file_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_upload_no_source_view_with_full_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_file_new
        )
        self.grant_access(
            obj=self.test_source, permission=permission_document_file_new
        )
        file_count = self.test_document.files.count()

        self._clear_events()

        response = self._request_document_file_upload_no_source_view()
        self.assertEqual(response.status_code, 302)

        self.test_document.refresh_from_db()
        self.assertEqual(self.test_document.files.count(), file_count + 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 4)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, test_document_file)
        self.assertEqual(events[0].verb, event_document_file_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_edited.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(events[2].target, test_document_version)
        self.assertEqual(events[2].verb, event_document_version_created.id)

        self.assertEqual(events[3].action_object, test_document_version)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, test_document_version_page)
        self.assertEqual(
            events[3].verb, event_document_version_page_created.id
        )

    def test_trashed_document_file_upload_no_source_view_with_full_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_file_new
        )
        self.grant_access(
            obj=self.test_source, permission=permission_document_file_new
        )
        file_count = self.test_document.files.count()

        self.test_document.delete()

        self._clear_events()

        response = self._request_document_file_upload_no_source_view()
        self.assertEqual(response.status_code, 404)

        self.test_document.refresh_from_db()
        self.assertEqual(self.test_document.files.count(), file_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



import logging

from django.db import models
from django.utils.translation import ugettext_lazy as _

from mayan.apps.databases.model_mixins import BackendModelMixin
from mayan.apps.documents.models.document_models import Document

from .classes import NullBackend
from .managers import (
    DuplicateBackendEntryManager, StoredDuplicateBackendManager
)

logger = logging.getLogger(name=__name__)


class StoredDuplicateBackend(BackendModelMixin, models.Model):
    _backend_model_null_backend = NullBackend

    objects = StoredDuplicateBackendManager()

    class Meta:
        verbose_name = _('Stored duplicate backend')
        verbose_name_plural = _('Stored duplicate backends')

    def __str__(self):
        return str(self.get_backend_label())


class DuplicateBackendEntry(models.Model):
    stored_backend = models.ForeignKey(
        on_delete=models.CASCADE, related_name='duplicate_entries',
        to=StoredDuplicateBackend, verbose_name=_('Stored duplicate backend')
    )
    document = models.ForeignKey(
        on_delete=models.CASCADE, related_name='duplicates', to=Document,
        verbose_name=_('Document')
    )
    documents = models.ManyToManyField(
        related_name='as_duplicate', to=Document, verbose_name=_(
            'Duplicated documents'
        )
    )

    objects = DuplicateBackendEntryManager()

    class Meta:
        unique_together = ('stored_backend', 'document')
        verbose_name = _('Duplicated backend entry')
        verbose_name_plural = _('Duplicated backend entries')


class DuplicateSourceDocument(Document):
    class Meta:
        proxy = True


class DuplicateTargetDocument(Document):
    class Meta:
        proxy = True



from shutil import copyfileobj

import magic

from mayan.apps.storage.utils import NamedTemporaryFile

from ..classes import MIMETypeBackend


class MIMETypeBackendPythonMagic(MIMETypeBackend):
    def _init(self, copy_length=None):
        self.copy_length = copy_length

    def _get_mime_type(self, file_object, mime_type_only):
        """
        Determine a file's MIME type by calling the system's libmagic
        library via python-magic.
        """
        file_mime_type = None
        file_mime_encoding = None

        with NamedTemporaryFile() as temporary_file_object:
            file_object.seek(0)
            copyfileobj(
                fsrc=file_object, fdst=temporary_file_object,
                length=self.copy_length
            )
            file_object.seek(0)
            temporary_file_object.seek(0)

            kwargs = {'mime': True}

            if not mime_type_only:
                kwargs['mime_encoding'] = True

            mime = magic.Magic(**kwargs)

            if mime_type_only:
                file_mime_type = mime.from_file(
                    filename=temporary_file_object.name
                )
            else:
                file_mime_type, file_mime_encoding = mime.from_file(
                    filename=temporary_file_object.name
                ).split('; charset=')

        return file_mime_type, file_mime_encoding



import mock

from mayan.apps.testing.tests.base import BaseTestCase
from mayan.apps.file_caching.models import CachePartition

from .mixins import AssetTestMixin


class AssetModelTestCase(
    AssetTestMixin, BaseTestCase
):
    def test_asset_get_absolute_url_method(self):
        self._create_test_asset()

        self.test_asset.get_absolute_url()

    def test_asset_generate_image_method(self):
        self._create_test_asset()

        with mock.patch.object(CachePartition, attribute='create_file') as mock_cache_partition_create_file:
            self.test_asset.generate_image()
            self.assertTrue(mock_cache_partition_create_file.called)

        self.test_asset.generate_image()

        with mock.patch.object(CachePartition, attribute='create_file') as mock_cache_partition_create_file:
            self.test_asset.generate_image()
            self.assertFalse(mock_cache_partition_create_file.called)



from django.db import migrations, models
import mayan.apps.common.validators


class Migration(migrations.Migration):

    dependencies = [
        ('metadata', '0016_auto_20211226_0905'),
    ]

    operations = [
        migrations.AddField(
            model_name='metadatatype', name='parser_arguments',
            field=models.TextField(
                blank=True, help_text='Enter the arguments for the '
                'parser in YAML format.', validators=[
                    mayan.apps.common.validators.YAMLValidator()
                ], verbose_name='Parser arguments'
            ),
        ),
        migrations.AlterField(
            model_name='metadatatype', name='parser',
            field=models.CharField(
                blank=True, help_text='The parser will reformat the '
                'value entered to conform to the expected format.',
                max_length=224, verbose_name='Parser'
            ),
        ),
        migrations.AlterField(
            model_name='metadatatype', name='validation',
            field=models.CharField(
                blank=True, help_text='The validator will reject '
                'data entry if the value entered does not conform to '
                'the expected format.', max_length=224,
                verbose_name='Validator'
            ),
        ),
    ]



import logging

import pycountry

from django.utils.translation import ugettext_lazy as _

from .settings import setting_language_codes

logger = logging.getLogger(name=__name__)


def get_language(language_code):
    language = getattr(
        pycountry.languages.get(alpha_3=language_code), 'name', None
    )

    if language:
        return _(language)
    else:
        return _('Unknown language "%s"') % language_code


def get_language_choices():
    result = []

    for iso639_3 in setting_language_codes.value:
        entry = pycountry.languages.get(alpha_3=iso639_3)
        if entry:
            label = _(entry.name)
            result.append(
                (iso639_3, label)
            )
        else:
            logger.warning('Unknown language code "%s".', iso639_3)

    return sorted(result, key=lambda x: x[1])



from django.db import models, migrations


class Migration(migrations.Migration):
    dependencies = [
        ('auth', '0001_initial'),
        ('permissions', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='permissionholder',
            name='holder_type',
        ),
        migrations.RemoveField(
            model_name='permissionholder',
            name='permission',
        ),
        migrations.DeleteModel(
            name='PermissionHolder',
        ),
        migrations.RemoveField(
            model_name='rolemember',
            name='member_type',
        ),
        migrations.RemoveField(
            model_name='rolemember',
            name='role',
        ),
        migrations.DeleteModel(
            name='RoleMember',
        ),
        migrations.AddField(
            model_name='role',
            name='groups',
            field=models.ManyToManyField(
                related_name='roles', verbose_name='Groups', to='auth.Group'
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='role',
            name='permissions',
            field=models.ManyToManyField(
                related_name='roles', verbose_name='Permissions',
                to='permissions.StoredPermission'
            ),
            preserve_default=True,
        ),
    ]



from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.search import (
    document_file_search, document_file_page_search, document_search,
    document_version_search, document_version_page_search
)

# Document

document_search.add_model_field(
    field='comments__text', label=_('Comments')
)

# Document file

document_file_search.add_model_field(
    field='document__comments__text',
    label=_('Document comments')
)

# Document file page

document_file_page_search.add_model_field(
    field='document_file__document__comments__text',
    label=_('Document comments')
)

# Document version

document_version_search.add_model_field(
    field='document__comments__text',
    label=_('Document comments')
)

# Document version page

document_version_page_search.add_model_field(
    field='document_version__document__comments__text',
    label=_('Document comments')
)



from rest_framework.mixins import CreateModelMixin


class CreateOnlyFieldSerializerMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not self.is_create_view():
            # Remove the create only fields if the view is anything other
            # than a create view.
            for field in getattr(self.Meta, 'create_only_fields', ()):
                # Use a default of None to avoid an error when trying to
                # remove a create only field already removed by the dynamic
                # fields mixin.
                self.fields.pop(field, None)

    def is_create_view(self):
        request = self.context.get('request')
        view = self.context.get('view')

        if request and view:
            if isinstance(view, CreateModelMixin) and request.method.lower() == 'post':
                # This is a create view with a request to create an instance.
                return True

        return False


class DynamicFieldListSerializerMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        fields_exclude = self.context.get('fields_exclude', '').split(',')
        fields_only = self.context.get('fields_only', '').split(',')

        self.fields_only = {None: ()}
        self.fields_only.update(
            self.group_fields(field_change_list=fields_only)
        )
        self.fields_exclude = {None: ()}
        self.fields_exclude.update(
            self.group_fields(field_change_list=fields_exclude)
        )

        self.update_fields(
            fields_exclude=self.fields_exclude[None],
            fields_only=self.fields_only[None]
        )

    def bind(self, field_name, parent):
        super().bind(field_name=field_name, parent=parent)

        self.update_fields(
            fields_exclude=getattr(
                self.root, 'fields_exclude', {}
            ).get(
                field_name, ()
            ),
            fields_only=getattr(
                self.root, 'fields_only', {}
            ).get(
                field_name, ()
            )
        )

    def group_fields(self, field_change_list, serializer_name=None):
        """
        Group all fields to be changed by object instance.
        Return a dictionary objects instance keys with their fields as
        sets.
        """
        serializers_fields = {}

        for field_change_name in field_change_list:
            if field_change_name:
                if '__' in field_change_name:
                    field_name, child_fields = field_change_name.split('__', 1)

                    related_serializer_fields = self.group_fields(
                        field_change_list=(child_fields,),
                        serializer_name=field_name
                    )

                    for key, value in related_serializer_fields.items():
                        serializers_fields.setdefault(key, set())
                        serializers_fields[key].update(value)
                else:
                    field_name = field_change_name

                serializers_fields.setdefault(serializer_name, set())
                serializers_fields[serializer_name].add(field_name)

        return serializers_fields

    def update_fields(self, fields_exclude=None, fields_only=None):
        fields_exclude = fields_exclude or ()
        fields_only = fields_only or ()

        if fields_only:
            for field_name in list(self.fields):
                if field_name not in fields_only:
                    self.fields.pop(field_name, None)

        for field_name in fields_exclude:
            if field_name not in self.fields_exclude:
                self.fields.pop(field_name, None)



from django.core.exceptions import PermissionDenied

from rest_framework.permissions import BasePermission

from mayan.apps.acls.models import AccessControlList
from mayan.apps.permissions import Permission


class MayanPermission(BasePermission):
    def has_permission(self, request, view):
        required_permissions = getattr(
            view, 'mayan_view_permissions', {}
        ).get(request.method, None)

        if required_permissions:
            try:
                Permission.check_user_permissions(
                    permissions=required_permissions, user=request.user
                )
            except PermissionDenied:
                return False
            else:
                return True
        else:
            return True

    def has_object_permission(self, request, view, obj):
        required_permissions = getattr(
            view, 'mayan_object_permissions', {}
        ).get(request.method, None)

        if required_permissions:
            try:
                AccessControlList.objects.check_access(
                    obj=obj, permissions=required_permissions,
                    user=request.user
                )
            except PermissionDenied:
                return False
            else:
                return True
        else:
            return True



from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from mayan.apps.authentication.link_conditions import condition_user_is_authenticated
from mayan.apps.navigation.classes import Link

from .icons import (
    icon_message_create, icon_message_delete, icon_message_list,
    icon_message_mark_read, icon_message_mark_read_all,
    icon_message_mark_unread
)
from .permissions import (
    permission_message_create, permission_message_delete,
    permission_message_view
)


def condition_is_read(context):
    return context['object'].read


def condition_is_unread(context):
    return not context['object'].read


def get_unread_message_count(context):
    Message = apps.get_model(
        app_label='messaging', model_name='Message'
    )
    if context.request.user.is_authenticated:
        return Message.objects.filter(
            user=context.request.user
        ).filter(read=False).count()


link_message_create = Link(
    icon=icon_message_create, permissions=(permission_message_create,),
    text=_('Create message'), view='messaging:message_create'
)
link_message_multiple_delete = Link(
    icon=icon_message_delete, tags='dangerous', text=_('Delete'),
    view='messaging:message_multiple_delete'
)
link_message_single_delete = Link(
    args='object.pk', icon=icon_message_delete,
    permissions=(permission_message_delete,),
    tags='dangerous', text=_('Delete'), view='messaging:message_single_delete'
)
link_message_list = Link(
    condition=condition_user_is_authenticated,
    badge_text=get_unread_message_count, icon=icon_message_list,
    text='', view='messaging:message_list'
)
link_message_single_mark_read = Link(
    args='object.pk', conditional_disable=condition_is_read,
    icon=icon_message_mark_read, text=_('Mark as read'),
    permissions=(permission_message_view,),
    view='messaging:message_single_mark_read'
)
link_message_single_mark_unread = Link(
    args='object.pk', conditional_disable=condition_is_unread,
    icon=icon_message_mark_unread, text=_('Mark as unread'),
    permissions=(permission_message_view,),
    view='messaging:message_single_mark_unread'
)
link_message_multiple_mark_read = Link(
    icon=icon_message_mark_read, text=_('Mark as read'),
    view='messaging:message_multiple_mark_read'
)
link_message_multiple_mark_unread = Link(
    icon=icon_message_mark_unread, text=_('Mark as unread'),
    view='messaging:message_multiple_mark_unread'
)
link_message_all_mark_read = Link(
    icon=icon_message_mark_read_all, text=_('Mark all as read'),
    view='messaging:message_all_mark_read'
)



from django.shortcuts import get_object_or_404

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.permissions import permission_document_type_view
from mayan.apps.documents.serializers.document_type_serializers import DocumentTypeSerializer
from mayan.apps.rest_api import generics
from mayan.apps.rest_api.api_view_mixins import (
    AsymmetricSerializerAPIViewMixin, ExternalObjectAPIViewMixin
)

from ..models import IndexTemplate
from ..permissions import (
    permission_index_template_create, permission_index_template_delete,
    permission_index_template_edit, permission_index_template_rebuild,
    permission_index_template_view
)
from ..serializers import (
    DocumentTypeAddSerializer, DocumentTypeRemoveSerializer,
    IndexTemplateSerializer, IndexTemplateNodeSerializer,
    IndexTemplateNodeWriteSerializer
)
from ..tasks import task_index_template_rebuild


class APIIndexTemplateListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the defined indexes template.
    post: Create a new index template.
    """
    mayan_object_permissions = {'GET': (permission_index_template_view,)}
    mayan_view_permissions = {'POST': (permission_index_template_create,)}
    ordering_fields = ('enabled', 'id', 'label', 'slug')
    queryset = IndexTemplate.objects.all()
    serializer_class = IndexTemplateSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }


class APIIndexTemplateDetailView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected index template.
    get: Returns the details of the selected index template.
    patch: Partially edit an index template.
    put: Edit an index template.
    """
    lookup_url_kwarg = 'index_template_id'
    mayan_object_permissions = {
        'GET': (permission_index_template_view,),
        'PUT': (permission_index_template_edit,),
        'PATCH': (permission_index_template_edit,),
        'DELETE': (permission_index_template_delete,)
    }
    queryset = IndexTemplate.objects.all()
    serializer_class = IndexTemplateSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }


class APIIndexTemplateDocumentTypeListView(
    ExternalObjectAPIViewMixin, generics.ListAPIView
):
    """
    get: Returns a list of the document types associated with this index template.
    """
    external_object_class = IndexTemplate
    external_object_pk_url_kwarg = 'index_template_id'
    mayan_external_object_permissions = {
        'GET': (permission_index_template_view,)
    }
    mayan_object_permissions = {'GET': (permission_document_type_view,)}
    serializer_class = DocumentTypeSerializer

    def get_queryset(self):
        return self.external_object.document_types.all()


class APIIndexTemplateDocumentTypeAddView(generics.ObjectActionAPIView):
    """
    post: Add a document type to an index template.
    """
    lookup_url_kwarg = 'index_template_id'
    mayan_object_permissions = {
        'POST': (permission_index_template_edit,)
    }
    serializer_class = DocumentTypeAddSerializer
    queryset = IndexTemplate.objects.all()

    def object_action(self, request, serializer):
        document_type = serializer.validated_data['document_type']
        self.object._event_actor = self.request.user
        self.object.document_types_add(queryset=[document_type])


class APIIndexTemplateDocumentTypeRemoveView(generics.ObjectActionAPIView):
    """
    post: Remove a document type from an index template.
    """
    lookup_url_kwarg = 'index_template_id'
    mayan_object_permissions = {
        'POST': (permission_index_template_edit,)
    }
    serializer_class = DocumentTypeRemoveSerializer
    queryset = IndexTemplate.objects.all()

    def object_action(self, request, serializer):
        document_type = serializer.validated_data['document_type']
        self.object._event_actor = self.request.user
        self.object.document_types_remove(queryset=[document_type])


class APIIndexTemplateNodeViewMixin(AsymmetricSerializerAPIViewMixin):
    object_permissions = {
        'GET': permission_index_template_view,
        'PATCH': permission_index_template_edit,
        'PUT': permission_index_template_edit,
        'POST': permission_index_template_edit,
        'DELETE': permission_index_template_edit
    }
    read_serializer_class = IndexTemplateNodeSerializer
    write_serializer_class = IndexTemplateNodeWriteSerializer

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super().get_serializer(*args, **kwargs)

    def get_index_template(self):
        if 'index_template_id' in self.kwargs:
            permission = self.object_permissions[self.request.method]

            queryset = AccessControlList.objects.restrict_queryset(
                permission=permission, queryset=IndexTemplate.objects.all(),
                user=self.request.user
            )

            return get_object_or_404(
                klass=queryset, pk=self.kwargs['index_template_id']
            )

    def get_serializer_context(self):
        context = super().get_serializer_context()
        context['index_template'] = self.get_index_template()
        return context


class APIIndexTemplateNodeListView(
    APIIndexTemplateNodeViewMixin, generics.ListCreateAPIView
):
    """
    get: Returns a list of all the template nodes for the selected index.
    post: Create a new index template node.
    """
    ordering_fields = ('enabled', 'id', 'link_documents')

    def get_queryset(self):
        return self.get_index_template().index_template_root_node.get_children()


class APIIndexTemplateNodeDetailView(
    APIIndexTemplateNodeViewMixin, generics.RetrieveUpdateDestroyAPIView
):
    """
    delete: Delete the selected index template node.
    get: Returns the details of the selected index template node.
    patch: Partially edit an index template node.
    put: Edit an index template node.
    """
    lookup_url_kwarg = 'index_template_node_id'

    def get_queryset(self):
        return self.get_index_template().index_template_nodes.all()


class APIIndexTemplateRebuildView(generics.ObjectActionAPIView):
    """
    post: Rebuild the selected index template.
    """
    lookup_url_kwarg = 'index_template_id'
    mayan_object_permissions = {
        'POST': (permission_index_template_rebuild,)
    }
    queryset = IndexTemplate.objects.all()

    def object_action(self, request, serializer):
        task_index_template_rebuild.apply_async(
            kwargs={
                'index_id': self.object.pk
            }
        )


class APIIndexTemplateResetView(generics.ObjectActionAPIView):
    """
    post: Reset the selected index template.
    """
    lookup_url_kwarg = 'index_template_id'
    mayan_object_permissions = {
        'POST': (permission_index_template_rebuild,)
    }
    queryset = IndexTemplate.objects.all()

    def object_action(self, request, serializer):
        self.object.reset()



from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import SettingNamespace

from .literals import (
    DEFAULT_LOGGING_ENABLE, DEFAULT_LOGGING_HANDLERS, DEFAULT_LOGGING_LEVEL,
    DEFAULT_LOGGING_LOG_FILE_PATH, LOGGING_HANDLER_OPTIONS
)

namespace = SettingNamespace(label=_('Logging'), name='logging')

setting_logging_enable = namespace.add_setting(
    default=DEFAULT_LOGGING_ENABLE, global_name='LOGGING_ENABLE', help_text=_(
        'Automatically enable logging to all apps.'
    )
)
setting_logging_handlers = namespace.add_setting(
    default=DEFAULT_LOGGING_HANDLERS, global_name='LOGGING_HANDLERS', help_text=_(
        'List of handlers to which logging messages will be sent. '
        'Options are: {}'.format(', '.join(LOGGING_HANDLER_OPTIONS))
    )
)
setting_logging_level = namespace.add_setting(
    default=DEFAULT_LOGGING_LEVEL, global_name='LOGGING_LEVEL', help_text=_(
        'Level for the logging system.'
    )
)
setting_logging_log_file_path = namespace.add_setting(
    default=DEFAULT_LOGGING_LOG_FILE_PATH,
    global_name='LOGGING_LOG_FILE_PATH', help_text=_(
        'Path to the logfile that will track errors during production.'
    ), is_path=True
)



from django.apps import apps
from django.db.models.signals import post_delete
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_list_facet, menu_multi_item, menu_tools
)
from mayan.apps.documents.menus import menu_documents
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.signals import signal_post_document_file_upload
from mayan.apps.navigation.classes import SourceColumn

from .classes import DuplicateBackend
from .handlers import (
    handler_remove_empty_duplicates_lists, handler_scan_duplicates_for
)
from .links import (
    link_document_duplicates_list, link_duplicated_document_list,
    link_duplicated_document_scan
)


class DuplicatesApp(MayanAppConfig):
    app_namespace = 'duplicates'
    app_url = 'duplicates'
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.duplicates'
    verbose_name = _('Duplicates')

    def ready(self):
        super().ready()

        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )

        DuplicateBackendEntry = self.get_model(
            model_name='DuplicateBackendEntry'
        )
        DuplicateSourceDocument = self.get_model(
            model_name='DuplicateSourceDocument'
        )
        DuplicateTargetDocument = self.get_model(
            model_name='DuplicateTargetDocument'
        )

        DuplicateBackend.load_modules()

        SourceColumn(
            func=lambda context: DuplicateBackendEntry.objects.get_duplicates_of(
                document=context['object'],
                permission=permission_document_view,
                user=context['request'].user
            ).count(), include_label=True, label=_('Duplicates'),
            order=99, source=DuplicateSourceDocument
        )

        SourceColumn(
            attribute='backend', include_label=True,
            label=_('Duplicate backend'), order=99,
            source=DuplicateTargetDocument
        )

        menu_documents.bind_links(
            links=(link_duplicated_document_list,)
        )
        menu_list_facet.bind_links(
            links=(link_document_duplicates_list,),
            sources=(Document,)
        )
        menu_tools.bind_links(
            links=(link_duplicated_document_scan,)
        )

        post_delete.connect(
            dispatch_uid='duplicates_handler_remove_empty_duplicates_lists',
            receiver=handler_remove_empty_duplicates_lists,
            sender=Document
        )
        signal_post_document_file_upload.connect(
            dispatch_uid='duplicates_handler_scan_duplicates_for',
            receiver=handler_scan_duplicates_for
        )



from django import forms

from mayan.apps.views.forms import DetailForm

from .models import Theme, UserThemeSetting


class ThemeForm(forms.ModelForm):
    class Meta:
        fields = ('label', 'stylesheet')
        model = Theme


class UserThemeSettingForm(forms.ModelForm):
    class Meta:
        fields = ('theme',)
        model = UserThemeSetting
        widgets = {
            'theme': forms.Select(
                attrs={
                    'class': 'select2'
                }
            ),
        }


class UserThemeSettingForm_view(DetailForm):
    class Meta:
        fields = ('theme',)
        model = UserThemeSetting



from django.conf.urls import url

from .views import (
    GlobalErrorLogEntryList, ObjectErrorLogEntryListClearView,
    ObjectErrorLogEntryListView
)

urlpatterns = [
    url(
        regex=r'^object/(?P<app_label>[-\w]+)/(?P<model_name>[-\w]+)/(?P<object_id>\d+)/errors/$',
        name='object_error_list', view=ObjectErrorLogEntryListView.as_view()
    ),
    url(
        regex=r'^object/(?P<app_label>[-\w]+)/(?P<model_name>[-\w]+)/(?P<object_id>\d+)/errors/clear/$',
        name='object_error_list_clear',
        view=ObjectErrorLogEntryListClearView.as_view()
    ),
    url(
        regex=r'^error_logs/partitions/entries/$',
        name='global_error_log_partition_entry_list',
        view=GlobalErrorLogEntryList.as_view()
    ),
]



from django.contrib import messages
from django.template import RequestContext
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from mayan.apps.views.generics import ConfirmView, SingleObjectListView

from ..icons import icon_user_notifications_list
from ..links import link_event_types_subscriptions_list

__all__ = (
    'NotificationListView', 'NotificationMarkRead', 'NotificationMarkReadAll'
)


class NotificationListView(SingleObjectListView):
    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_user_notifications_list,
            'no_results_main_link': link_event_types_subscriptions_list.resolve(
                context=RequestContext(request=self.request)
            ),
            'no_results_text': _(
                'Subscribe to global or object events to receive '
                'notifications.'
            ),
            'no_results_title': _('There are no notifications'),
            'object': self.request.user,
            'title': _('Notifications'),
        }

    def get_source_queryset(self):
        return self.request.user.notifications.all()


class NotificationMarkRead(ConfirmView):
    post_action_redirect = reverse_lazy(
        viewname='events:user_notifications_list'
    )

    def get_extra_context(self):
        return {
            'title': _('Mark the selected notification as read?')
        }

    def get_queryset(self):
        return self.request.user.notifications.all()

    def view_action(self, form=None):
        self.get_queryset().filter(
            pk=self.kwargs['notification_id']
        ).update(read=True)

        messages.success(
            message=_('Notification marked as read.'), request=self.request
        )


class NotificationMarkReadAll(ConfirmView):
    post_action_redirect = reverse_lazy(
        viewname='events:user_notifications_list'
    )

    def get_extra_context(self):
        return {
            'title': _('Mark all notification as read?')
        }

    def get_queryset(self):
        return self.request.user.notifications.all()

    def view_action(self, form=None):
        self.get_queryset().update(read=True)

        messages.success(
            message=_('All notifications marked as read.'),
            request=self.request
        )



from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend

from mayan.apps.databases.classes import BaseBackend


class DjangoAuthenticationBackendModelEmail(ModelBackend):
    def authenticate(self, request, username=None, password=None, **kwargs):
        UserModel = get_user_model()

        if username is None or password is None:
            return
        try:
            user = UserModel._default_manager.get(email=username)
        except UserModel.DoesNotExist:
            # Execute the default password hasher to avoid valid user
            # discovery via differential timming analysis.
            UserModel().set_password(raw_password=password)
        else:
            if user.check_password(raw_password=password) and self.user_can_authenticate(user=user):
                return user


class DjangoAuthenticationBackendWrapperMultiFactor(BaseBackend):
    factors = ()
    _default_factor = None
    _factor_instance = None
    _factors_map = {}

    def _get_factor(self, factor_name=None):
        if not factor_name:
            return self._default_factor
        else:
            return self._factors_map[factor_name]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        for factor_entry in self.factors:
            factor_name = factor_entry['name']
            if factor_entry.get('default', False):
                self._default_factor = factor_entry

            self._factors_map[factor_name] = factor_entry

    def authenticate(self, request, **kwargs):
        factor_name = kwargs.pop('factor_name', None)

        factor = self._get_factor(factor_name=factor_name)
        factor_instance = factor['class']()

        return factor_instance.authenticate(request, **kwargs)

    def get_user(self, user_id, **kwargs):
        factor_name = kwargs.pop('factor_name', None)

        factor = self._get_factor(factor_name=factor_name)
        factor_instance = factor['class']()

        return factor_instance.get_user(user_id=user_id)



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('logging', '0005_auto_20220119_0518')
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='errorlogpartition',
            unique_together={
                ('error_log', 'content_type', 'object_id'),
                ('error_log', 'name')
            }
        )
    ]



import dbm
import logging
import os
from pathlib import Path
import shutil
import tempfile

from django.apps import apps
from django.utils.module_loading import import_string

from .classes import DefinedStorage, PassthroughStorage
from .settings import setting_temporary_directory

logger = logging.getLogger(name=__name__)


def NamedTemporaryFile(*args, **kwargs):
    kwargs.update({'dir': setting_temporary_directory.value})
    return tempfile.NamedTemporaryFile(*args, **kwargs)


class PassthroughStorageProcessor:
    def __init__(
        self, app_label, defined_storage_name, log_file, model_name,
        file_attribute='file'
    ):
        self.app_label = app_label
        self.defined_storage_name = defined_storage_name
        self.file_attribute = file_attribute
        self.log_file = log_file
        self.model_name = model_name

    def _update_entry(self, key):
        if not self.reverse:
            self.database[key] = '1'
        else:
            try:
                del self.database[key]
            except KeyError:
                pass

    def _inclusion_condition(self, key):
        if self.reverse:
            return key in self.database
        else:
            return key not in self.database

    def execute(self, reverse=False):
        self.reverse = reverse
        model = apps.get_model(
            app_label=self.app_label, model_name=self.model_name
        )

        storage_instance = DefinedStorage.get(
            name=self.defined_storage_name
        ).get_storage_instance()

        if isinstance(storage_instance, PassthroughStorage):
            ContentType = apps.get_model(
                app_label='contenttypes', model_name='ContentType'
            )
            content_type = ContentType.objects.get_for_model(model=model)

            self.database = dbm.open(file=self.log_file, flag='c')

            for instance in model.objects.all():
                key = '{}.{}'.format(content_type.name, instance.pk)
                if self._inclusion_condition(key=key):
                    file_name = getattr(instance, self.file_attribute).name

                    content = storage_instance.open(
                        name=file_name, mode='rb',
                        _direct=not self.reverse
                    )
                    storage_instance.delete(name=file_name)
                    storage_instance.save(
                        name=file_name, content=content,
                        _direct=self.reverse
                    )
                    self._update_entry(key=key)

            self.database.close


def TemporaryDirectory(*args, **kwargs):
    kwargs.update({'dir': setting_temporary_directory.value})
    return tempfile.TemporaryDirectory(*args, **kwargs)


def TemporaryFile(*args, **kwargs):
    kwargs.update({'dir': setting_temporary_directory.value})
    return tempfile.TemporaryFile(*args, **kwargs)


def fs_cleanup(filename, suppress_exceptions=True):
    """
    Tries to remove the given filename. Ignores non-existent files.
    """
    try:
        os.remove(filename)
    except OSError:
        try:
            shutil.rmtree(path=filename)
        except OSError:
            if suppress_exceptions:
                """Ignore exception."""
            else:
                raise


def get_storage_subclass(dotted_path):
    """
    Import a storage class and return a subclass that will always return eq
    True to avoid creating a new migration when for runtime storage class
    changes. Used now only by historic migrations.
    """
    imported_storage_class = import_string(dotted_path=dotted_path)

    class StorageSubclass(imported_storage_class):
        def __init__(self, *args, **kwargs):
            return super().__init__(*args, **kwargs)

        def __eq__(self, other):
            return True

        def deconstruct(self):
            return ('mayan.apps.storage.classes.FakeStorageSubclass', (), {})

    return StorageSubclass


def mkdtemp(*args, **kwargs):
    """
    Creates a temporary directory in the most secure manner possible.
    There are no race conditions in the directory's creation.
    The directory is readable, writable, and searchable only by the creating
    user ID.
    """
    kwargs.update({'dir': setting_temporary_directory.value})
    return tempfile.mkdtemp(*args, **kwargs)


def patch_files(path=None, replace_list=None):
    """
    Search and replace content from a list of file based on a pattern
    replace_list[
        {
            'filename_pattern': '*.css',
            'content_patterns': [
                {
                    'search': '',
                    'replace': '',
                }
            ]
        }
    ]
    """
    file_open_mode = 'r+'

    path_object = Path(path)
    for replace_entry in replace_list or []:
        for path_entry in path_object.glob('**/{}'.format(replace_entry['filename_pattern'])):
            if path_entry.is_file():
                for pattern in replace_entry['content_patterns']:
                    with path_entry.open(mode=file_open_mode) as source_file_object:
                        with tempfile.TemporaryFile(mode=file_open_mode) as temporary_file_object:
                            source_position = 0
                            destination_position = 0

                            while(True):
                                source_file_object.seek(source_position)
                                letter = source_file_object.read(1)

                                if len(letter) == 0:
                                    break
                                else:
                                    if letter == pattern['search'][0]:
                                        text = '{}{}'.format(letter, source_file_object.read(len(pattern['search']) - 1))

                                        temporary_file_object.seek(destination_position)
                                        if text == pattern['search']:
                                            text = pattern['replace']
                                            source_position = source_position + len(pattern['search'])
                                            destination_position = destination_position + len(pattern['replace'])
                                            temporary_file_object.write(text)

                                        else:
                                            source_position = source_position + 1
                                            destination_position = destination_position + 1
                                            temporary_file_object.write(letter)
                                    else:
                                        source_position = source_position + 1
                                        destination_position = destination_position + 1
                                        temporary_file_object.write(letter)

                            source_file_object.seek(0)
                            source_file_object.truncate()
                            temporary_file_object.seek(0)
                            shutil.copyfileobj(
                                fsrc=temporary_file_object,
                                fdst=source_file_object
                            )


def touch(filename, times=None):
    with open(file=filename, mode='a'):
        os.utime(filename, times)



from django.contrib import messages
from django.utils.translation import ugettext_lazy as _

from actstream.models import Action

from mayan.apps.common.classes import QuerysetParametersSerializer
from mayan.apps.views.generics import ConfirmView
from mayan.apps.views.mixins import ExternalContentTypeObjectViewMixin

from ..permissions import permission_events_clear
from ..tasks import task_event_queryset_clear

from .mixins import EventViewMixin

__all__ = (
    'EventListClearView', 'ObjectEventClearView', 'VerbEventClearView'
)


class EventClearBaseView(ConfirmView):
    object_permission = permission_events_clear

    def get_extra_context(self):
        return {
            'message': _(
                'This action is not reversible. The process will be performed '
                'in the background. '
            )
        }

    def get_task_extra_kwargs(self):
        return {}

    def view_action(self):
        decomposed_queryset = QuerysetParametersSerializer.decompose(
            _model=Action, **self.get_queryset_parameters()
        )

        task_kwargs = {
            'decomposed_queryset': decomposed_queryset,
            'user_id': self.request.user.pk
        }

        task_kwargs.update(self.get_task_extra_kwargs())

        task_event_queryset_clear.apply_async(kwargs=task_kwargs)

        messages.success(
            request=self.request, message=_(
                'Event list clear task queued successfully.'
            )
        )


class EventListClearView(EventClearBaseView):
    object_permission = permission_events_clear

    def get_extra_context(self):
        context = super().get_extra_context()
        context.update(
            {
                'title': _('Clear events'),
            }
        )
        return context

    def get_queryset_parameters(self):
        return {
            '_method_name': 'all'
        }


class ObjectEventClearView(
    ExternalContentTypeObjectViewMixin, EventClearBaseView
):
    def get_extra_context(self):
        context = super().get_extra_context()
        context.update(
            {
                'object': self.external_object,
                'title': _('Clear events of: %s') % self.external_object,
            }
        )
        return context

    def get_queryset_parameters(self):
        return {
            '_method_name': 'any', 'obj': self.external_object
        }

    def get_task_extra_kwargs(self):
        return {
            'target_content_type_id': self.external_object_content_type.pk,
            'target_object_id': self.external_object.pk
        }


class VerbEventClearView(EventViewMixin, EventClearBaseView):
    def get_extra_context(self):
        context = super().get_extra_context()
        context.update(
            {
                'title': _(
                    'Clear events of type: %s'
                ) % self.get_event_type(id=self.kwargs['verb'])
            }
        )
        return context

    def get_queryset_parameters(self):
        return {
            '_method_name': 'filter', 'verb': self.kwargs['verb']
        }



from django import forms

from ..fields import DocumentVersionField
from ..models.document_version_models import DocumentVersion

__all__ = ('DocumentVersionForm', 'DocumentVersionPreviewForm',)


class DocumentVersionForm(forms.ModelForm):
    class Meta:
        fields = ('comment',)
        model = DocumentVersion


class DocumentVersionPreviewForm(forms.Form):
    def __init__(self, *args, **kwargs):
        document_version = kwargs.pop('instance', None)
        transformation_instance_list = kwargs.pop(
            'transformation_instance_list', ()
        )
        super().__init__(*args, **kwargs)
        self.fields['document_version'].initial = document_version
        self.fields['document_version'].widget.attrs.update(
            {
                'transformation_instance_list': transformation_instance_list
            }
        )

    document_version = DocumentVersionField()



from django.db import migrations, models

import mayan.apps.documents.models.document_file_models
import mayan.apps.storage.classes


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0061_documentversionpage'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='documentfilepage',
            name='enabled',
        ),
        migrations.AlterField(
            model_name='documentfile',
            name='file',
            field=models.FileField(
                storage=mayan.apps.storage.classes.DefinedStorageLazy(
                    name='documents__documentfiles'
                ), upload_to=mayan.apps.documents.models.document_file_models.upload_to,
                verbose_name='File'
            ),
        ),
        migrations.AlterField(
            model_name='documentversionpage',
            name='page_number',
            field=models.PositiveIntegerField(
                db_index=True, default=1, editable=False,
                verbose_name='Page number'
            ),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0065_documentversion_active'),
    ]

    operations = [
        migrations.AddField(
            model_name='documentfile',
            name='filename',
            field=models.CharField(
                default='', max_length=255, verbose_name='Filename'
            ),
            preserve_default=False,
        ),
    ]



from django.apps import apps
from django.db.models.signals import post_delete, post_save, pre_delete
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.acls.permissions import permission_acl_edit, permission_acl_view
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.classes import ModelCopy
from mayan.apps.common.menus import (
    menu_list_facet, menu_main, menu_object, menu_related, menu_secondary,
    menu_setup, menu_tools
)
from mayan.apps.documents.links.document_type_links import link_document_type_list
from mayan.apps.documents.signals import signal_post_initial_document_type
from mayan.apps.events.classes import EventModelRegistry, ModelEventType
from mayan.apps.navigation.classes import SourceColumn
from mayan.apps.rest_api.fields import DynamicSerializerField
from mayan.apps.views.html_widgets import TwoStateWidget

from .events import event_index_template_created, event_index_template_edited
from .handlers import (
    handler_create_default_document_index, handler_delete_empty,
    handler_index_document, handler_remove_document
)
from .html_widgets import (
    get_instance_link, index_instance_item_link, node_level
)
from .links import (
    link_document_index_instance_list, link_document_type_index_templates,
    link_index_instance_menu, link_index_instance_rebuild,
    link_index_instances_reset, link_index_template_setup,
    link_index_template_create, link_index_template_document_types,
    link_index_template_delete, link_index_template_edit,
    link_index_template_list, link_index_template_node_tree_view,
    link_index_instances_rebuild, link_index_template_node_create,
    link_index_template_node_delete, link_index_template_node_edit
)
from .permissions import (
    permission_index_template_delete, permission_index_template_edit,
    permission_index_instance_view,
    permission_index_template_rebuild, permission_index_template_view
)


class DocumentIndexingApp(MayanAppConfig):
    app_namespace = 'indexing'
    app_url = 'indexing'
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.document_indexing'
    verbose_name = _('Document indexing')

    def ready(self):
        super().ready()

        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )

        DocumentType = apps.get_model(
            app_label='documents', model_name='DocumentType'
        )

        DocumentIndexInstanceNode = self.get_model(
            model_name='DocumentIndexInstanceNode'
        )

        IndexInstance = self.get_model(model_name='IndexInstance')
        IndexInstanceNode = self.get_model(model_name='IndexInstanceNode')
        IndexInstanceNodeSearchResult = self.get_model(
            model_name='IndexInstanceNodeSearchResult'
        )
        IndexTemplate = self.get_model(model_name='IndexTemplate')
        IndexTemplateNode = self.get_model(model_name='IndexTemplateNode')

        DynamicSerializerField.add_serializer(
            klass=IndexTemplate,
            serializer_class='mayan.apps.document_indexing.serializers.IndexTemplateSerializer'
        )

        EventModelRegistry.register(model=IndexTemplate)

        ModelCopy(
            model=IndexTemplateNode, excludes={'parent__isnull': False},
            extra_kwargs={'get_or_create': True}
        ).add_fields(
            field_names=(
                'index', 'expression', 'enabled', 'link_documents'
            ),
        )
        ModelCopy(
            model=IndexTemplate, bind_link=True, register_permission=True
        ).add_fields(
            field_names=(
                'label', 'slug', 'enabled', 'document_types',
                'index_template_nodes'
            ),
        )

        ModelEventType.register(
            event_types=(
                event_index_template_created, event_index_template_edited
            ), model=IndexTemplate
        )

        ModelPermission.register(
            model=Document, permissions=(
                permission_index_instance_view,
            )
        )
        ModelPermission.register(
            model=IndexTemplate, permissions=(
                permission_acl_edit, permission_acl_view,
                permission_index_template_delete,
                permission_index_template_edit,
                permission_index_instance_view,
                permission_index_template_rebuild,
                permission_index_template_view,
            )
        )
        ModelPermission.register_inheritance(
            model=IndexTemplateNode, related='index'
        )

        ModelPermission.register_inheritance(
            model=IndexInstanceNode, related='index_template_node__index'
        )

        # Document Index Instance Node

        SourceColumn(
            func=lambda context: get_instance_link(
                index_instance_node=context['object'],
            ), include_label=True, is_sortable=True, label=_('Level'),
            sort_field='value', source=DocumentIndexInstanceNode
        )

        # Index instance

        SourceColumn(
            attribute='get_level_count', include_label=True,
            label=_('Depth'), source=IndexInstance
        )
        SourceColumn(
            attribute='get_descendants_count', include_label=True,
            label=_('Total nodes'), source=IndexInstance
        )
        SourceColumn(
            func=lambda context: context[
                'object'
            ].get_descendants_document_count(
                user=context['request'].user
            ), include_label=True, label=_('Total documents'),
            help_text=_(
                'Number of unique documents this item contains.'
            ), source=IndexInstance
        )

        # Index instance node

        column_index_instance_node_level = SourceColumn(
            func=lambda context: index_instance_item_link(context['object']),
            is_identifier=True, is_sortable=True, label=_('Level'),
            sort_field='value', source=IndexInstanceNode
        )
        column_index_instance_node_level.add_exclude(
            source=DocumentIndexInstanceNode
        )

        column_index_instance_node_level_count = SourceColumn(
            attribute='get_level_count', include_label=True,
            label=_('Depth'), source=IndexInstanceNode
        )
        column_index_instance_node_level_count.add_exclude(
            source=DocumentIndexInstanceNode
        )

        column_index_instance_node_count = SourceColumn(
            attribute='get_descendants_count',
            include_label=True, label=_('Nodes'), source=IndexInstanceNode
        )
        column_index_instance_node_count.add_exclude(
            source=DocumentIndexInstanceNode
        )

        column_index_instance_node_document_count = SourceColumn(
            func=lambda context: context[
                'object'
            ].get_descendants_document_count(
                user=context['request'].user
            ), include_label=True, label=_('Documents'),
            help_text=_(
                'Number of unique documents this item contains.'
            ), source=IndexInstanceNode
        )
        column_index_instance_node_document_count.add_exclude(
            source=DocumentIndexInstanceNode
        )

        SourceColumn(
            func=lambda context: index_instance_item_link(context['object']),
            is_identifier=True, is_sortable=True, label=_('Level'),
            sort_field='value', source=IndexInstanceNodeSearchResult
        )
        SourceColumn(
            attribute='get_full_path', source=IndexInstanceNodeSearchResult
        )

        # Index template

        column_index_label = SourceColumn(
            attribute='label', is_identifier=True, is_sortable=True,
            source=IndexTemplate
        )
        column_index_label.add_exclude(source=IndexInstance)
        SourceColumn(
            attribute='label', is_object_absolute_url=True,
            is_identifier=True, is_sortable=True, source=IndexInstance
        )
        column_index_slug = SourceColumn(
            attribute='slug', include_label=True, is_sortable=True,
            source=IndexTemplate
        )
        column_index_slug.add_exclude(source=IndexInstance)
        column_index_enabled = SourceColumn(
            attribute='enabled', include_label=True, is_sortable=True,
            source=IndexTemplate, widget=TwoStateWidget
        )
        column_index_enabled.add_exclude(source=IndexInstance)

        # Index template node

        SourceColumn(
            func=lambda context: node_level(context['object']),
            include_label=True, is_identifier=True, label=_('Level'),
            source=IndexTemplateNode
        )
        SourceColumn(
            attribute='enabled', include_label=True, is_sortable=True,
            source=IndexTemplateNode, widget=TwoStateWidget
        )
        SourceColumn(
            attribute='link_documents', include_label=True, is_sortable=True,
            source=IndexTemplateNode, widget=TwoStateWidget
        )

        menu_list_facet.bind_links(
            links=(
                link_document_index_instance_list,
            ), sources=(Document,)
        )
        menu_list_facet.bind_links(
            links=(link_document_type_index_templates,),
            sources=(DocumentType,)
        )
        menu_list_facet.bind_links(
            links=(
                link_index_template_document_types,
                link_index_template_node_tree_view
            ), sources=(IndexTemplate,)
        )
        menu_object.bind_links(
            links=(
                link_index_template_delete, link_index_template_edit,
                link_index_instance_rebuild
            ), sources=(IndexTemplate,)
        )
        menu_object.bind_links(
            links=(
                link_index_template_node_create, link_index_template_node_edit,
                link_index_template_node_delete
            ), sources=(IndexTemplateNode,)
        )
        menu_main.bind_links(links=(link_index_instance_menu,), position=50)
        menu_related.bind_links(
            links=(link_index_template_list,),
            sources=(
                DocumentType, 'documents:document_type_list',
                'documents:document_type_create'
            )
        )
        menu_related.bind_links(
            links=(link_document_type_list,),
            sources=(
                IndexTemplate, 'indexing:index_template_list',
                'indexing:index_template_create'
            )
        )
        menu_secondary.bind_links(
            links=(link_index_template_list, link_index_template_create),
            sources=(
                IndexTemplate, 'indexing:index_template_list',
                'indexing:index_template_create'
            )
        )
        menu_setup.bind_links(links=(link_index_template_setup,))
        menu_tools.bind_links(
            links=(link_index_instances_rebuild, link_index_instances_reset)
        )

        post_save.connect(
            dispatch_uid='document_indexing_handler_index_document',
            receiver=handler_index_document,
            sender=Document
        )
        post_delete.connect(
            dispatch_uid='document_indexing_handler_delete_empty',
            receiver=handler_delete_empty,
            sender=Document
        )
        pre_delete.connect(
            dispatch_uid='document_indexing_handler_remove_document',
            receiver=handler_remove_document,
            sender=Document
        )
        signal_post_initial_document_type.connect(
            dispatch_uid='document_indexing_handler_create_default_document_index',
            receiver=handler_create_default_document_index,
            sender=DocumentType
        )



from django.contrib import admin

from .models.document_models import Document
from .models.document_file_models import DocumentFile
from .models.document_file_page_models import DocumentFilePage
from .models.document_type_models import DocumentType, DocumentTypeFilename
from .models.recently_accessed_document_models import RecentlyAccessedDocument
from .models.trashed_document_models import TrashedDocument


class DocumentFilePageInline(admin.StackedInline):
    model = DocumentFilePage
    extra = 1
    classes = ('collapse-open',)
    allow_add = True


class DocumentTypeFilenameInline(admin.StackedInline):
    model = DocumentTypeFilename
    extra = 1
    classes = ('collapse-open',)
    allow_add = True


class DocumentFileInline(admin.StackedInline):
    model = DocumentFile
    extra = 1
    classes = ('collapse-open',)
    allow_add = True


@admin.register(Document)
class DocumentAdmin(admin.ModelAdmin):
    date_hierarchy = 'datetime_created'
    inlines = (DocumentFileInline,)
    list_filter = ('document_type', 'is_stub')
    list_display = ('uuid', 'label', 'document_type', 'datetime_created', 'is_stub')
    readonly_fields = ('uuid', 'document_type', 'datetime_created')


@admin.register(DocumentType)
class DocumentTypeAdmin(admin.ModelAdmin):
    inlines = (DocumentTypeFilenameInline,)
    list_display = (
        'label', 'trash_time_period', 'trash_time_unit', 'delete_time_period',
        'delete_time_unit'
    )


@admin.register(RecentlyAccessedDocument)
class RecentlyAccessedDocumentAdmin(admin.ModelAdmin):
    date_hierarchy = 'datetime_accessed'
    list_display = ('user', 'document', 'datetime_accessed')
    list_display_links = ('document', 'datetime_accessed')
    list_filter = ('user',)
    readonly_fields = ('user', 'document', 'datetime_accessed')


@admin.register(TrashedDocument)
class TrashedDocumentAdmin(admin.ModelAdmin):
    date_hierarchy = 'trashed_date_time'
    list_filter = ('document_type',)
    list_display = ('uuid', 'label', 'document_type', 'trashed_date_time')
    readonly_fields = ('uuid', 'document_type')



from django.contrib.auth.models import Group
from django.template import RequestContext
from django.urls import reverse_lazy
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from mayan.apps.user_management.permissions import permission_group_edit
from mayan.apps.views.generics import (
    AddRemoveView, MultipleObjectDeleteView, SingleObjectCreateView,
    SingleObjectDetailView, SingleObjectEditView, SingleObjectListView
)

from .forms import StoredPermissionDetailForm
from .icons import icon_role_list
from .links import link_role_create
from .models import Role, StoredPermission
from .permissions import (
    permission_role_view, permission_role_create, permission_role_delete,
    permission_role_edit
)


class GroupRolesView(AddRemoveView):
    main_object_method_add_name = 'roles_add'
    main_object_method_remove_name = 'roles_remove'
    main_object_model = Group
    main_object_permission = permission_group_edit
    main_object_pk_url_kwarg = 'group_id'
    secondary_object_model = Role
    secondary_object_permission = permission_role_edit
    list_available_title = _('Available roles')
    list_added_title = _('Group roles')
    related_field = 'roles'

    def get_actions_extra_kwargs(self):
        return {'_event_actor': self.request.user}

    def get_extra_context(self):
        return {
            'object': self.main_object,
            'title': _('Roles of group: %s') % self.main_object,
        }


class RoleCreateView(SingleObjectCreateView):
    fields = ('label',)
    model = Role
    view_permission = permission_role_create
    post_action_redirect = reverse_lazy(viewname='permissions:role_list')

    def get_extra_context(self):
        return {'title': _('Create new role')}

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}


class RoleDeleteView(MultipleObjectDeleteView):
    error_message = _('Error deleting role "%(instance)s"; %(exception)s')
    model = Role
    object_permission = permission_role_delete
    pk_url_kwarg = 'role_id'
    post_action_redirect = reverse_lazy(viewname='permissions:role_list')
    success_message_single = _('Role "%(object)s" deleted successfully.')
    success_message_singular = _('%(count)d role deleted successfully.')
    success_message_plural = _('%(count)d roles deleted successfully.')
    title_single = _('Delete role: %(object)s.')
    title_singular = _('Delete the %(count)d selected role.')
    title_plural = _('Delete the %(count)d selected roles.')


class RoleEditView(SingleObjectEditView):
    fields = ('label',)
    model = Role
    object_permission = permission_role_edit
    pk_url_kwarg = 'role_id'

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}


class RoleListView(SingleObjectListView):
    model = Role
    object_permission = permission_role_view

    def get_extra_context(self):
        return {
            'hide_link': True,
            'hide_object': True,
            'no_results_icon': icon_role_list,
            'no_results_main_link': link_role_create.resolve(
                context=RequestContext(request=self.request)
            ),
            'no_results_text': _(
                'Roles are authorization units. They contain '
                'user groups which inherit the role permissions for the '
                'entire system. Roles can also part of access '
                'controls lists. Access controls list are permissions '
                'granted to a role for specific objects which its group '
                'members inherit.'
            ),
            'no_results_title': _('There are no roles'),
            'title': _('Roles'),
        }


class RoleMembersView(AddRemoveView):
    main_object_method_add_name = 'groups_add'
    main_object_method_remove_name = 'groups_remove'
    main_object_model = Role
    main_object_permission = permission_role_edit
    main_object_pk_url_kwarg = 'role_id'
    secondary_object_model = Group
    secondary_object_permission = permission_group_edit
    list_available_title = _('Available groups')
    list_added_title = _('Role groups')
    related_field = 'groups'

    def get_actions_extra_kwargs(self):
        return {'_event_actor': self.request.user}

    def get_extra_context(self):
        return {
            'object': self.main_object,
            'title': _('Groups of role: %s') % self.main_object,
            'subtitle': _(
                'Add groups to be part of a role. They will '
                'inherit the role\'s permissions and access controls.'
            ),
        }


class RolePermissionsView(AddRemoveView):
    grouped = True
    main_object_method_add_name = 'permissions_add'
    main_object_method_remove_name = 'permissions_remove'
    main_object_model = Role
    main_object_permission = permission_role_edit
    main_object_pk_url_kwarg = 'role_id'
    list_available_title = _('Available permissions')
    list_added_title = _('Granted permissions')
    related_field = 'permissions'
    secondary_object_model = StoredPermission

    def generate_choices(self, queryset):
        namespaces_dictionary = {}

        # Sort permissions by their translatable label.
        object_list = sorted(
            queryset,
            key=lambda permission: permission.volatile_permission.label
        )

        # Group permissions by namespace.
        for permission in object_list:
            namespaces_dictionary.setdefault(
                permission.volatile_permission.namespace.label,
                []
            )
            namespaces_dictionary[
                permission.volatile_permission.namespace.label
            ].append(
                (permission.pk, force_text(s=permission))
            )

        # Sort permissions by their translatable namespace label.
        return sorted(namespaces_dictionary.items())

    def get_actions_extra_kwargs(self):
        return {'_event_actor': self.request.user}

    def get_extra_context(self):
        return {
            'object': self.main_object,
            'subtitle': _(
                'Permissions granted here will apply to the entire system '
                'and all objects.'
            ),
            'title': _('Permissions for role: %s') % self.main_object,
        }


class StoredPermissionDetailView(SingleObjectDetailView):
    form_class = StoredPermissionDetailForm
    form_extra_kwargs = {
        'extra_fields': [
            {
                'field': 'volatile_permission'
            },
        ]
    }
    model = StoredPermission
    pk_url_kwarg = 'stored_permission_id'

    def get_extra_context(self, **kwargs):
        return {
            'object': self.object,
            'title': _('Details of permission: %s') % self.object
        }



# Generated by Django 2.2.12 on 2020-06-16 07:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mailer', '0006_auto_20191213_0044'),
    ]

    operations = [
        migrations.DeleteModel(
            name='LogEntry',
        ),
        migrations.DeleteModel(
            name='UserMailerLogEntry',
        ),
    ]



from mayan.apps.views.http import URL

from .settings import setting_organization_installation_url


def get_organization_installation_url(request=None):
    result = setting_organization_installation_url.value
    if not result and request:
        result = URL(
            netloc=request.get_host(), port=request.get_port(),
            scheme=request.scheme
        ).to_string()

    return result



import os

from django.conf import settings

from .runtime import celery_class

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'mayan.settings.production')

app = celery_class('mayan')
app.config_from_object('django.conf:settings', namespace='CELERY')
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)



import logging

from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from mayan.apps.storage.models import SharedUploadedFile

from ..classes import SourceBackend
from ..forms import WebFormUploadFormHTML5

from .mixins import (
    SourceBackendCompressedMixin, SourceBackendInteractiveMixin,
    SourceBaseMixin
)

__all__ = ('SourceBackendWebForm',)
logger = logging.getLogger(name=__name__)


class SourceBackendWebForm(
    SourceBackendCompressedMixin, SourceBackendInteractiveMixin,
    SourceBaseMixin, SourceBackend
):
    label = _('Web form')
    upload_form_class = WebFormUploadFormHTML5

    def get_view_context(self, context, request):
        return {
            'subtemplates_list': [
                {
                    'name': 'appearance/generic_multiform_subtemplate.html',
                    'context': {
                        'forms': context['forms'],
                        'is_multipart': True,
                        'form_action': '{}?{}'.format(
                            reverse(
                                viewname=request.resolver_match.view_name,
                                kwargs=request.resolver_match.kwargs
                            ), request.META['QUERY_STRING']
                        ),
                        'form_css_classes': 'dropzone',
                        'form_disable_submit': True,
                        'form_id': 'html5upload',
                    },
                }
            ]
        }

    def get_shared_uploaded_files(self):
        return (
            SharedUploadedFile.objects.create(
                file=self.process_kwargs['forms']['source_form'].cleaned_data['file']
            ),
        )



from django.utils.translation import ugettext_lazy as _

DJANGO_SQLITE_BACKEND = 'django.db.backends.sqlite3'

IMPORT_ERROR_EXCLUSION_TEXTS = (
    'doesn\'t look like a module path', 'No module named'
)

MESSAGE_SQLITE_WARNING = _(
    'Your database backend is set to use SQLite. SQLite should only be used '
    'for development and testing, not for production.'
)
